@@ -25,6 +25,14 @@
 #include <QtCore/QCoreApplication>
 #include <QtCore/QTimer>
 
+#include <QtGui/QApplication>
+#include <QtGui/QWidget>
+
+#include <QtDBus/QDBusConnection>
+#include <QtDBus/QDBusConnectionInterface>
+
+#include <kdebug.h>
+
 #include <PolkitQt1/Authority>
 #include <PolkitQt1/Subject>
 
@@ -55,7 +63,7 @@ Polkit1Backend::Polkit1Backend()
     : AuthBackend()
     , m_flyingActions(false)
 {
-    setCapabilities(AuthorizeFromHelperCapability | CheckActionExistenceCapability);
+    setCapabilities(AuthorizeFromHelperCapability | CheckActionExistenceCapability | PreAuthActionCapability);
 
     // Setup useful signals
     connect(PolkitQt1::Authority::instance(), SIGNAL(configChanged()),
@@ -66,8 +74,8 @@ Polkit1Backend::Polkit1Backend()
             this, SLOT(updateCachedActions(PolkitQt1::ActionDescription::List)));
 
     // Cache existing action IDs as soon as possible
-    PolkitQt1::Authority::instance()->enumerateActions();
     m_flyingActions = true;
+    PolkitQt1::Authority::instance()->enumerateActions();
 }
 
 Polkit1Backend::~Polkit1Backend()
@@ -75,11 +83,49 @@ Polkit1Backend::~Polkit1Backend()
 
 }
 
+void Polkit1Backend::preAuthAction(const QString& action, QWidget* parent)
+{
+    kDebug();
+    // If a parent was not specified, skip this
+    if (!parent) {
+        kDebug() << "Parent widget does not exist, skipping";
+        return;
+    }
+
+    // Are we running our KDE auth agent?
+    if (QDBusConnection::sessionBus().interface()->isServiceRegistered(QLatin1String("org.kde.Polkit1AuthAgent"))) {
+        // Check if we actually are entitled to use GUI capabilities
+        if (qApp == 0 || QApplication::type() == QApplication::Tty) {
+            kDebug() << "Not streaming parent as we are on a TTY application";
+        }
+
+        // Retrieve the dialog root window Id
+        qulonglong wId = parent->effectiveWinId();
+
+        // Send it over the bus to our agent
+        QDBusMessage methodCall =
+                QDBusMessage::createMethodCall(QLatin1String("org.kde.Polkit1AuthAgent"), QLatin1String("/org/kde/Polkit1AuthAgent"), QLatin1String("org.kde.Polkit1AuthAgent"),
+                                               QLatin1String("setWIdForAction"));
+
+        methodCall << action;
+        methodCall << wId;
+
+        QDBusPendingCall call = QDBusConnection::sessionBus().asyncCall(methodCall);
+        call.waitForFinished();
+
+        if (call.isError()) {
+            kWarning() << "ERROR while streaming the parent!!" << call.error();
+        }
+    } else {
+        kDebug() << "KDE polkit agent appears too old or not registered on the bus";
+    }
+}
+
 void Polkit1Backend::updateCachedActions(const PolkitQt1::ActionDescription::List& actions)
 {
     m_knownActions.clear();
-    foreach (PolkitQt1::ActionDescription *action, actions) {
-        m_knownActions << action->actionId();
+    foreach (const PolkitQt1::ActionDescription& action, actions) {
+        m_knownActions << action.actionId();
     }
     m_flyingActions = false;
 }
@@ -99,7 +145,7 @@ void Polkit1Backend::setupAction(const QString &action)
 Action::AuthStatus Polkit1Backend::actionStatus(const QString &action)
 {
     PolkitQt1::UnixProcessSubject subject(QCoreApplication::applicationPid());
-    PolkitQt1::Authority::Result r = PolkitQt1::Authority::instance()->checkAuthorizationSync(action, &subject,
+    PolkitQt1::Authority::Result r = PolkitQt1::Authority::instance()->checkAuthorizationSync(action, subject,
                                                                                               PolkitQt1::Authority::None);
     switch (r) {
     case PolkitQt1::Authority::Yes:
@@ -134,7 +180,7 @@ bool Polkit1Backend::isCallerAuthorized(const QString &action, QByteArray caller
     PolkitResultEventLoop e;
     connect(authority, SIGNAL(checkAuthorizationFinished(PolkitQt1::Authority::Result)),
             &e, SLOT(requestQuit(PolkitQt1::Authority::Result)));
-    authority->checkAuthorization(action, &subject, PolkitQt1::Authority::AllowUserInteraction);
+    authority->checkAuthorization(action, subject, PolkitQt1::Authority::AllowUserInteraction);
     e.exec();
 
     switch (e.result()) {
