@@ -1195,6 +1195,7 @@ KCoreConfigSkeleton::ItemLongLong *KCoreConfigSkeleton::addItemLongLong( const Q
   return item;
 }
 
+#ifndef KDE_NO_DEPRECATED
 KCoreConfigSkeleton::ItemLongLong *KCoreConfigSkeleton::addItemInt64(
         const QString& name,
         qint64 &reference,
@@ -1203,6 +1204,7 @@ KCoreConfigSkeleton::ItemLongLong *KCoreConfigSkeleton::addItemInt64(
 {
     return addItemLongLong(name, reference, defaultValue, key);
 }
+#endif
 
 KCoreConfigSkeleton::ItemULongLong *KCoreConfigSkeleton::addItemULongLong( const QString &name, quint64 &reference,
                                      quint64 defaultValue, const QString &key )
@@ -1214,6 +1216,7 @@ KCoreConfigSkeleton::ItemULongLong *KCoreConfigSkeleton::addItemULongLong( const
   return item;
 }
 
+#ifndef KDE_NO_DEPRECATED
 KCoreConfigSkeleton::ItemULongLong *KCoreConfigSkeleton::addItemUInt64(
         const QString & name,
         quint64 &reference,
@@ -1222,6 +1225,7 @@ KCoreConfigSkeleton::ItemULongLong *KCoreConfigSkeleton::addItemUInt64(
 {
     return addItemULongLong(name, reference, defaultValue, key);
 }
+#endif
 
 KCoreConfigSkeleton::ItemDouble *KCoreConfigSkeleton::addItemDouble( const QString &name, double &reference,
                                      double defaultValue, const QString &key )
