@@ -1059,8 +1059,8 @@ class KSharedDataCache::Private
         // mutex support (systemSupportsProcessSharing), then we:
         // Open the file and resize to some sane value if the file is too small.
         if (file.open(QIODevice::ReadWrite) &&
-           (file.size() >= size || file.resize(size)) &&
-           ensureFileAllocated(file.handle(), size))
+            (file.size() >= size ||
+             (file.resize(size) && ensureFileAllocated(file.handle(), size))))
         {
             // Use mmap directly instead of QFile::map since the QFile (and its
             // shared mapping) will disappear unless we hang onto the QFile for no
