@@ -22,8 +22,8 @@
 #include "kdedadaptor.h"
 #include "kdedmodule.h"
 
-#include "kresourcelist.h"
 #include <kcrash.h>
+#include <kdeversion.h>
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -40,6 +40,11 @@
 #include <kapplication.h>
 #include <kcmdlineargs.h>
 #include <kaboutdata.h>
+#ifndef KDE_NO_DEPRECATED
+#include <klibloader.h>
+#else
+#include <klibrary.h>
+#endif
 #include <klocale.h>
 #include <kglobal.h>
 #include <kconfig.h>
@@ -71,7 +76,11 @@ static bool bCheckSycoca;
 static bool bCheckUpdates;
 static bool bCheckHostname;
 
-extern void qDBusAddSpyHook(void (*)(const QDBusMessage&));
+#ifdef Q_DBUS_EXPORT
+extern Q_DBUS_EXPORT void qDBusAddSpyHook(void (*)(const QDBusMessage&));
+#else
+extern QDBUS_EXPORT void qDBusAddSpyHook(void (*)(const QDBusMessage&));
+#endif
 
 static void runBuildSycoca(QObject *callBackObj=0, const char *callBackSlot=0)
 {
@@ -200,37 +209,58 @@ void Kded::messageFilter(const QDBusMessage &message)
   Q_UNUSED(module);
 }
 
+static int phaseForModule(const KService::Ptr& service)
+{
+    const QVariant phasev = service->property("X-KDE-Kded-phase", QVariant::Int );
+    return phasev.isValid() ? phasev.toInt() : 2;
+}
+
 void Kded::initModules()
 {
-     m_dontLoad.clear();
-     bool kde_running = !qgetenv( "KDE_FULL_SESSION" ).isEmpty();
-    // not the same user like the one running the session (most likely we're run via sudo or something)
-    const QByteArray sessionUID = qgetenv( "KDE_SESSION_UID" );
-    if( !sessionUID.isEmpty() && uid_t( sessionUID.toInt() ) != getuid())
-        kde_running = false;
+    m_dontLoad.clear();
+    bool kde_running = !qgetenv( "KDE_FULL_SESSION" ).isEmpty();
+    if (kde_running) {
+        // not the same user like the one running the session (most likely we're run via sudo or something)
+        const QByteArray sessionUID = qgetenv( "KDE_SESSION_UID" );
+        if( !sessionUID.isEmpty() && uid_t( sessionUID.toInt() ) != getuid())
+            kde_running = false;
+
+        // not the same kde version as the current desktop
+        const QByteArray kdeSession = qgetenv("KDE_SESSION_VERSION");
+        if (kdeSession.toInt() != KDE_VERSION_MAJOR)
+            kde_running = false;
+    }
+
+    // There will be a "phase 2" only if we're in the KDE startup.
+    // If kded is restarted by its crashhandled or by hand,
+    // then there will be no second phase autoload, so load
+    // these modules now, if in a KDE session.
+    const bool loadPhase2Now = (kde_running && qgetenv("KDED_STARTED_BY_KDEINIT").toInt() == 0);
+
      // Preload kded modules.
      const KService::List kdedModules = KServiceTypeTrader::self()->query("KDEDModule");
      for(KService::List::ConstIterator it = kdedModules.begin(); it != kdedModules.end(); ++it)
      {
          KService::Ptr service = *it;
          // Should the service load on startup?
-         bool autoload = isModuleAutoloaded(service);
+         const bool autoload = isModuleAutoloaded(service);
 
          // see ksmserver's README for description of the phases
-         QVariant phasev = service->property("X-KDE-Kded-phase", QVariant::Int );
-         int phase = phasev.isValid() ? phasev.toInt() : 2;
          bool prevent_autoload = false;
-         switch( phase )
+         switch( phaseForModule(service) )
          {
              case 0: // always autoload
                  break;
              case 1: // autoload only in KDE
-                 if( !kde_running )
+                 if (!kde_running) {
                      prevent_autoload = true;
+                 }
                  break;
              case 2: // autoload delayed, only in KDE
              default:
-                 prevent_autoload = true;
+                 if (!loadPhase2Now) {
+                     prevent_autoload = true;
+                 }
                  break;
          }
 
@@ -255,20 +285,17 @@ void Kded::initModules()
 
 void Kded::loadSecondPhase()
 {
-     kDebug(7020) << "Loading second phase autoload";
-     KSharedConfig::Ptr config = KGlobal::config();
-     KService::List kdedModules = KServiceTypeTrader::self()->query("KDEDModule");
-     for(KService::List::ConstIterator it = kdedModules.constBegin(); it != kdedModules.constEnd(); ++it)
-     {
-         KService::Ptr service = *it;
-         bool autoload = service->property("X-KDE-Kded-autoload", QVariant::Bool).toBool();
-         KConfigGroup cg(config, QString("Module-%1").arg(service->desktopEntryName()));
-         autoload = cg.readEntry("autoload", autoload);
-         QVariant phasev = service->property("X-KDE-Kded-phase", QVariant::Int );
-         int phase = phasev.isValid() ? phasev.toInt() : 2;
-         if( phase == 2 && autoload )
+    kDebug(7020) << "Loading second phase autoload";
+    KSharedConfig::Ptr config = KGlobal::config();
+    KService::List kdedModules = KServiceTypeTrader::self()->query("KDEDModule");
+    for(KService::List::ConstIterator it = kdedModules.constBegin(); it != kdedModules.constEnd(); ++it) {
+        const KService::Ptr service = *it;
+        const bool autoload = isModuleAutoloaded(service);
+        if (autoload && phaseForModule(service) == 2) {
+            //kDebug(7020) << "2nd phase: loading" << service->desktopEntryName();
             loadModule(service, false);
-     }
+        }
+    }
 }
 
 void Kded::noDemandLoad(const QString &obj)
@@ -365,6 +392,7 @@ KDEDModule *Kded::loadModule(const KService::Ptr& s, bool onDemand)
             if (factoryName.isEmpty())
                 factoryName = s->library();
             factoryName = "create_" + factoryName;
+#ifndef KDE_NO_DEPRECATED
             KLibrary* lib = KLibLoader::self()->library(libname);
             KDEDModule* (*create)();
             if (lib) {
@@ -372,6 +400,7 @@ KDEDModule *Kded::loadModule(const KService::Ptr& s, bool onDemand)
                 if (create)
                     module = create();
             }
+#endif
             if (!module) {
                 kWarning() << "Could not load library" << libname << ". ["
                            << loader.errorString() << "]";
@@ -384,7 +413,7 @@ KDEDModule *Kded::loadModule(const KService::Ptr& s, bool onDemand)
             module->setModuleName(obj);
             m_modules.insert(obj, module);
             //m_libs.insert(obj, lib);
-            connect(module, SIGNAL(moduleDeleted(KDEDModule *)), SLOT(slotKDEDModuleRemoved(KDEDModule *)));
+            connect(module, SIGNAL(moduleDeleted(KDEDModule*)), SLOT(slotKDEDModuleRemoved(KDEDModule*)));
             kDebug(7020) << "Successfully loaded module" << obj;
             return module;
         } else {
@@ -449,12 +478,12 @@ void Kded::updateDirWatch()
   delete m_pDirWatch;
   m_pDirWatch = new KDirWatch;
 
-  QObject::connect( m_pDirWatch, SIGNAL(dirty(const QString&)),
-           this, SLOT(update(const QString&)));
-  QObject::connect( m_pDirWatch, SIGNAL(created(const QString&)),
-           this, SLOT(update(const QString&)));
-  QObject::connect( m_pDirWatch, SIGNAL(deleted(const QString&)),
-           this, SLOT(dirDeleted(const QString&)));
+  QObject::connect( m_pDirWatch, SIGNAL(dirty(QString)),
+           this, SLOT(update(QString)));
+  QObject::connect( m_pDirWatch, SIGNAL(created(QString)),
+           this, SLOT(update(QString)));
+  QObject::connect( m_pDirWatch, SIGNAL(deleted(QString)),
+           this, SLOT(dirDeleted(QString)));
 
   // For each resource
   for( QStringList::ConstIterator it = m_allResourceDirs.constBegin();
@@ -686,7 +715,7 @@ KUpdateD::KUpdateD()
     m_pTimer = new QTimer;
     m_pTimer->setSingleShot( true );
     connect(m_pTimer, SIGNAL(timeout()), this, SLOT(runKonfUpdate()));
-    QObject::connect( m_pDirWatch, SIGNAL(dirty(const QString&)),
+    QObject::connect( m_pDirWatch, SIGNAL(dirty(QString)),
            this, SLOT(slotNewUpdateFile()));
 
     const QStringList dirs = KGlobal::dirs()->findDirs("data", "kconf_update");
@@ -829,7 +858,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
 {
      KAboutData aboutData( "kded" /*don't change this one to kded4! dbus registration should be org.kde.kded etc.*/,
         "kdelibs4", ki18n("KDE Daemon"),
-        "$Id: kded.cpp 1156841 2010-07-29 19:59:05Z zander $",
+        KDE_VERSION_STRING,
         ki18n("KDE Daemon - triggers Sycoca database updates when needed"));
 
      KCmdLineOptions options;
@@ -842,7 +871,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
      KCmdLineArgs::addCmdLineOptions( options );
 
      // WABA: Make sure not to enable session management.
-     putenv(strdup("SESSION_MANAGER="));
+     putenv(qstrdup("SESSION_MANAGER="));
 
      // Parse command line before checking DCOP
      KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
@@ -879,7 +908,9 @@ extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
 
      Kded *kded = new Kded(); // Build data base
 
+#ifndef _WIN32_WCE
      KDE_signal(SIGTERM, sighandler);
+#endif
      KDE_signal(SIGHUP, sighandler);
      KDEDApplication k;
      k.setQuitOnLastWindowClosed(false);
