@@ -253,6 +253,7 @@ KWebPage::KWebPage(QObject *parent, Integration flags)
     setActionIcon(action(Paste), KIcon("edit-paste"));
     setActionIcon(action(Undo), KIcon("edit-undo"));
     setActionIcon(action(Redo), KIcon("edit-redo"));
+    setActionIcon(action(SelectAll), KIcon("edit-select-all"));
     setActionIcon(action(InspectElement), KIcon("view-process-all"));
     setActionIcon(action(OpenLinkInNewWindow), KIcon("window-new"));
     setActionIcon(action(OpenFrameInNewWindow), KIcon("window-new"));
@@ -324,8 +325,6 @@ void KWebPage::downloadRequest(const QNetworkRequest& request)
     KIO::TransferJob* job = KIO::get(request.url());
     connect(job, SIGNAL(mimetype(KIO::Job*,QString)),
             this, SLOT(_k_receivedContentType(KIO::Job*,QString)));
-    connect(job, SIGNAL(result(KJob*)),
-            this, SLOT(_k_receivedContentTypeResult(KJob*)));
 
     job->setMetaData(request.attribute(static_cast<QNetworkRequest::Attribute>(KIO::AccessManager::MetaData)).toMap());
     job->addMetaData(QL1S("MaxCacheSize"), QL1S("0")); // Don't store in http cache.
