@@ -232,9 +232,7 @@ public:
     void _k_slotCanceled();
     void _k_slotRedirected(const KUrl&);
     void _k_slotProperties();
-    void _k_slotPressed(const QModelIndex&);
     void _k_slotActivated(const QModelIndex&);
-    void _k_slotDoubleClicked(const QModelIndex&);
     void _k_slotSelectionChanged();
     void _k_openContextMenu(const QPoint&);
     void _k_triggerPreview(const QModelIndex&);
@@ -285,7 +283,6 @@ public:
     KUrl previewUrl;
     int previewWidth;
 
-    bool leftButtonPressed;
     bool dirHighlighting;
     bool onlyDoubleClickSelectsFiles;
     QString lastURL; // used for highlighting a directory on cdUp
@@ -325,7 +322,6 @@ KDirOperator::Private::Private(KDirOperator *_parent) :
     preview(0),
     previewUrl(),
     previewWidth(0),
-    leftButtonPressed(false),
     dirHighlighting(false),
     onlyDoubleClickSelectsFiles(!KGlobalSettings::singleClick()),
     progressDelayTimer(0),
@@ -378,8 +374,8 @@ KDirOperator::KDirOperator(const KUrl& _url, QWidget *parent) :
 {
     d->splitter = new QSplitter(this);
     d->splitter->setChildrenCollapsible(false);
-    connect(d->splitter, SIGNAL(splitterMoved(int, int)),
-            this, SLOT(_k_slotSplitterMoved(int, int)));
+    connect(d->splitter, SIGNAL(splitterMoved(int,int)),
+            this, SLOT(_k_slotSplitterMoved(int,int)));
 
     d->preview = 0;
 
@@ -389,9 +385,7 @@ KDirOperator::KDirOperator(const KUrl& _url, QWidget *parent) :
     if (_url.isEmpty()) { // no dir specified -> current dir
         QString strPath = QDir::currentPath();
         strPath.append(QChar('/'));
-        d->currUrl = KUrl();
-        d->currUrl.setProtocol(QLatin1String("file"));
-        d->currUrl.setPath(strPath);
+        d->currUrl = QUrl::fromLocalFile(strPath);
     } else {
         d->currUrl = _url;
         if (d->currUrl.protocol().isEmpty())
@@ -406,8 +400,8 @@ KDirOperator::KDirOperator(const KUrl& _url, QWidget *parent) :
     setLayoutDirection(Qt::LeftToRight);
     setDirLister(new KDirLister());
 
-    connect(&d->completion, SIGNAL(match(const QString&)),
-            SLOT(slotCompletionMatch(const QString&)));
+    connect(&d->completion, SIGNAL(match(QString)),
+            SLOT(slotCompletionMatch(QString)));
 
     d->progressBar = new QProgressBar(this);
     d->progressBar->setObjectName("d->progressBar");
@@ -762,8 +756,8 @@ bool KDirOperator::mkdir(const QString& directory, bool enterDirectory)
 
     for (; it != dirs.end(); ++it) {
         url.addPath(*it);
-        exists = KIO::NetAccess::exists(url, KIO::NetAccess::DestinationSide, 0);
-        writeOk = !exists && KIO::NetAccess::mkdir(url, topLevelWidget());
+        exists = KIO::NetAccess::exists(url, KIO::NetAccess::DestinationSide, this);
+        writeOk = !exists && KIO::NetAccess::mkdir(url, this);
     }
 
     if (exists) { // url was already existent
@@ -824,7 +818,7 @@ KIO::DeleteJob * KDirOperator::del(const KFileItemList& items,
     if (doIt) {
         KIO::JobFlags flags = showProgress ? KIO::DefaultFlags : KIO::HideProgressInfo;
         KIO::DeleteJob *job = KIO::del(urls, flags);
-        job->ui()->setWindow(topLevelWidget());
+        job->ui()->setWindow(this);
         job->ui()->setAutoErrorHandlingEnabled(true);
         return job;
     }
@@ -882,7 +876,7 @@ KIO::CopyJob * KDirOperator::trash(const KFileItemList& items,
     if (doIt) {
         KIO::JobFlags flags = showProgress ? KIO::DefaultFlags : KIO::HideProgressInfo;
         KIO::CopyJob *job = KIO::trash(urls, flags);
-        job->ui()->setWindow(topLevelWidget());
+        job->ui()->setWindow(this);
         job->ui()->setAutoErrorHandlingEnabled(true);
         return job;
     }
@@ -1038,17 +1032,11 @@ void KDirOperator::setUrl(const KUrl& _newurl, bool clearforward)
     KUrl newurl;
 
     if (!_newurl.isValid())
-        newurl.setPath(QDir::homePath());
+        newurl = QUrl::fromLocalFile(QDir::homePath());
     else
         newurl = _newurl;
 
     newurl.adjustPath( KUrl::AddTrailingSlash );
-#ifdef Q_WS_WIN
-    QString pathstr = QDir::fromNativeSeparators(newurl.toLocalFile());
-#else
-    QString pathstr = newurl.path();
-#endif
-    newurl.setPath(pathstr);
 
     // already set
     if (newurl.equals(d->currUrl, KUrl::CompareWithoutTrailingSlash))
@@ -1262,8 +1250,7 @@ void KDirOperator::cdUp()
 
 void KDirOperator::home()
 {
-    KUrl u;
-    u.setPath(QDir::homePath());
+    KUrl u = QUrl::fromLocalFile(QDir::homePath());
     setUrl(u, true);
 }
 
@@ -1588,20 +1575,16 @@ void KDirOperator::setView(QAbstractItemView *view)
     if (treeView) {
         QHeaderView* headerView = treeView->header();
         headerView->setSortIndicator(d->sortColumn(), d->sortOrder());
-        connect(headerView, SIGNAL(sortIndicatorChanged (int, Qt::SortOrder)),
-                this, SLOT(_k_synchronizeSortingState(int, Qt::SortOrder)));
+        connect(headerView, SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)),
+                this, SLOT(_k_synchronizeSortingState(int,Qt::SortOrder)));
     }
 
-    connect(d->itemView, SIGNAL(activated(const QModelIndex&)),
-            this, SLOT(_k_slotActivated(const QModelIndex&)));
-    connect(d->itemView, SIGNAL(doubleClicked(const QModelIndex&)),
-            this, SLOT(_k_slotDoubleClicked(const QModelIndex&)));
-    connect(d->itemView, SIGNAL(pressed(const QModelIndex&)),
-            this, SLOT(_k_slotPressed(const QModelIndex&)));
-    connect(d->itemView, SIGNAL(customContextMenuRequested(const QPoint&)),
-            this, SLOT(_k_openContextMenu(const QPoint&)));
-    connect(d->itemView, SIGNAL(entered(const QModelIndex&)),
-            this, SLOT(_k_triggerPreview(const QModelIndex&)));
+    connect(d->itemView, SIGNAL(activated(QModelIndex)),
+            this, SLOT(_k_slotActivated(QModelIndex)));
+    connect(d->itemView, SIGNAL(customContextMenuRequested(QPoint)),
+            this, SLOT(_k_openContextMenu(QPoint)));
+    connect(d->itemView, SIGNAL(entered(QModelIndex)),
+            this, SLOT(_k_triggerPreview(QModelIndex)));
 
     updateViewActions();
     d->splitter->insertWidget(0, d->itemView);
@@ -1620,10 +1603,10 @@ void KDirOperator::setView(QAbstractItemView *view)
     }
 
     connect(d->itemView->selectionModel(),
-            SIGNAL(currentChanged(const QModelIndex&,const QModelIndex&)),
-            this, SLOT(_k_triggerPreview(const QModelIndex&)));
+            SIGNAL(currentChanged(QModelIndex,QModelIndex)),
+            this, SLOT(_k_triggerPreview(QModelIndex)));
     connect(d->itemView->selectionModel(),
-            SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)),
+            SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
             this, SLOT(_k_slotSelectionChanged()));
 
     // if we cannot cast it to a QListView, disable the "Icon Position" menu. Note that this check
@@ -1694,14 +1677,14 @@ void KDirOperator::setDirLister(KDirLister *lister)
 
     connect(d->dirLister, SIGNAL(percent(int)),
             SLOT(_k_slotProgress(int)));
-    connect(d->dirLister, SIGNAL(started(const KUrl&)), SLOT(_k_slotStarted()));
+    connect(d->dirLister, SIGNAL(started(KUrl)), SLOT(_k_slotStarted()));
     connect(d->dirLister, SIGNAL(completed()), SLOT(_k_slotIOFinished()));
     connect(d->dirLister, SIGNAL(canceled()), SLOT(_k_slotCanceled()));
-    connect(d->dirLister, SIGNAL(redirection(const KUrl&)),
-            SLOT(_k_slotRedirected(const KUrl&)));
-    connect(d->dirLister, SIGNAL(newItems(const KFileItemList&)), SLOT(_k_slotItemsChanged()));
-    connect(d->dirLister, SIGNAL(itemsDeleted(const KFileItemList&)), SLOT(_k_slotItemsChanged()));
-    connect(d->dirLister, SIGNAL(itemsFilteredByMime(const KFileItemList&)), SLOT(_k_slotItemsChanged()));
+    connect(d->dirLister, SIGNAL(redirection(KUrl)),
+            SLOT(_k_slotRedirected(KUrl)));
+    connect(d->dirLister, SIGNAL(newItems(KFileItemList)), SLOT(_k_slotItemsChanged()));
+    connect(d->dirLister, SIGNAL(itemsDeleted(KFileItemList)), SLOT(_k_slotItemsChanged()));
+    connect(d->dirLister, SIGNAL(itemsFilteredByMime(KFileItemList)), SLOT(_k_slotItemsChanged()));
     connect(d->dirLister, SIGNAL(clear()), SLOT(_k_slotItemsChanged()));
 }
 
@@ -1951,17 +1934,17 @@ void KDirOperator::setupActions()
     KToggleAction *detailedAction = new KToggleAction(i18n("Detailed View"), this);
     d->actionCollection->addAction("detailed view", detailedAction);
     detailedAction->setIcon(KIcon(QLatin1String("view-list-details")));
-    connect(detailedAction, SIGNAL(triggered ()), SLOT(_k_slotDetailedView()));
+    connect(detailedAction, SIGNAL(triggered()), SLOT(_k_slotDetailedView()));
 
     KToggleAction *treeAction = new KToggleAction(i18n("Tree View"), this);
     d->actionCollection->addAction("tree view", treeAction);
     treeAction->setIcon(KIcon(QLatin1String("view-list-tree")));
-    connect(treeAction, SIGNAL(triggered ()), SLOT(_k_slotTreeView()));
+    connect(treeAction, SIGNAL(triggered()), SLOT(_k_slotTreeView()));
 
     KToggleAction *detailedTreeAction = new KToggleAction(i18n("Detailed Tree View"), this);
     d->actionCollection->addAction("detailed tree view", detailedTreeAction);
     detailedTreeAction->setIcon(KIcon(QLatin1String("view-list-tree")));
-    connect(detailedTreeAction, SIGNAL(triggered ()), SLOT(_k_slotDetailedTreeView()));
+    connect(detailedTreeAction, SIGNAL(triggered()), SLOT(_k_slotDetailedTreeView()));
 
     QActionGroup* viewGroup = new QActionGroup(this);
     shortAction->setActionGroup(viewGroup);
@@ -2007,7 +1990,7 @@ void KDirOperator::setupActions()
 
     d->newFileMenu = new KNewFileMenu(d->actionCollection, "new", this);
     connect(d->newFileMenu, SIGNAL(directoryCreated(KUrl)), this, SLOT(_k_slotDirectoryCreated(KUrl)));
-    
+
     d->actionCollection->addAssociatedWidget(this);
     foreach (QAction* action, d->actionCollection->actions())
       action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
@@ -2357,16 +2340,6 @@ void KDirOperator::Private::_k_slotProperties()
     }
 }
 
-void KDirOperator::Private::_k_slotPressed(const QModelIndex&)
-{
-    // Remember whether the left mouse button has been pressed, to prevent
-    // that a right-click on an item opens an item (see _k_slotDoubleClicked() and
-    // _k_openContextMenu()).
-    const Qt::KeyboardModifiers modifiers = QApplication::keyboardModifiers();
-    leftButtonPressed = (QApplication::mouseButtons() & Qt::LeftButton) &&
-                        !(modifiers & Qt::ShiftModifier) && !(modifiers & Qt::ControlModifier);
-}
-
 void KDirOperator::Private::_k_slotActivated(const QModelIndex& index)
 {
     const QModelIndex dirIndex = proxyModel->mapToSource(index);
@@ -2383,25 +2356,6 @@ void KDirOperator::Private::_k_slotActivated(const QModelIndex& index)
     }
 }
 
-void KDirOperator::Private::_k_slotDoubleClicked(const QModelIndex& index)
-{
-    if (!leftButtonPressed) {
-        return;
-    }
-
-    const QModelIndex dirIndex = proxyModel->mapToSource(index);
-    KFileItem item = dirModel->itemForIndex(dirIndex);
-
-    if (item.isNull())
-        return;
-
-    if (item.isDir()) {
-        parent->selectDir(item);
-    } else {
-        parent->selectFile(item);
-    }
-}
-
 void KDirOperator::Private::_k_slotSelectionChanged()
 {
     if (itemView == 0) {
@@ -2425,8 +2379,6 @@ void KDirOperator::Private::_k_slotSelectionChanged()
 
 void KDirOperator::Private::_k_openContextMenu(const QPoint& pos)
 {
-    leftButtonPressed = false;
-
     const QModelIndex proxyIndex = itemView->indexAt(pos);
     const QModelIndex dirIndex = proxyModel->mapToSource(proxyIndex);
     KFileItem item = dirModel->itemForIndex(dirIndex);
@@ -2565,7 +2517,7 @@ void KDirOperator::Private::_k_slotExpandToUrl(const QModelIndex &index)
             const KUrl url = *it;
             if (url.isParentOf(item.url())) {
                 const KFileItem _item = dirLister->findByUrl(url);
-                if (_item.isDir()) {
+                if (!_item.isNull() && _item.isDir()) {
                     const QModelIndex _index = dirModel->indexForItem(_item);
                     const QModelIndex _proxyIndex = proxyModel->mapFromSource(_index);
                     treeView->expand(_proxyIndex);
