@@ -737,6 +737,11 @@ void CanvasImageData::mark()
         data->mark();
 }
 
+JSObject* CanvasImageData::valueClone(Interpreter* targetCtx) const
+{
+    return static_cast<JSObject*>(getWrapper<CanvasImageData>(targetCtx->globalExec(), impl()->clone()));
+}
+
 const ClassInfo CanvasImageDataArray::info = { "ImageDataArray", 0, 0, 0 };
 
 CanvasImageDataArray::CanvasImageDataArray(ExecState* exec, CanvasImageData* p) :
@@ -832,7 +837,9 @@ void CanvasImageDataArray::put(ExecState* exec, unsigned index, JSValue* value,
         return;
     }
 
-    JSObject::put(exec, index, value, attr);
+    // Must use the string version here since numberic one will fall back to
+    // us again.
+    JSObject::put(exec, Identifier::from(index), value, attr);
 }
 
 DOM::CanvasImageDataImpl* toCanvasImageData(ExecState* exec, JSValue* val)
