@@ -33,6 +33,10 @@
 #include "misc/loader.h"
 #include "dom/html_form.h"
 #include "dom/html_image.h"
+#include "dom/dom_string.h"
+#include "dom/html_document.h"
+#include "dom/dom_element.h"
+#include "xml/dom_elementimpl.h"
 #include <QtGui/QClipboard>
 #include <QtCore/QFileInfo>
 #include <QtGui/QMenu>
@@ -64,8 +68,6 @@
 #include <kactioncollection.h>
 #include <kactionmenu.h>
 
-#include "dom/dom_element.h"
-
 #include "khtmlpart_p.h"
 
 KHTMLPartBrowserExtension::KHTMLPartBrowserExtension( KHTMLPart *parent )
@@ -110,12 +112,12 @@ void KHTMLPartBrowserExtension::editableWidgetFocused( QWidget *widget )
 
     if ( !m_connectedToClipboard && m_editableFormWidget )
     {
-        connect( QApplication::clipboard(), SIGNAL( dataChanged() ),
-                 this, SLOT( updateEditActions() ) );
+        connect( QApplication::clipboard(), SIGNAL(dataChanged()),
+                 this, SLOT(updateEditActions()) );
 
         if ( m_editableFormWidget->inherits( "QLineEdit" ) || m_editableFormWidget->inherits( "QTextEdit" ) )
-            connect( m_editableFormWidget, SIGNAL( selectionChanged() ),
-                     this, SLOT( updateEditActions() ) );
+            connect( m_editableFormWidget, SIGNAL(selectionChanged()),
+                     this, SLOT(updateEditActions()) );
 
         m_connectedToClipboard = true;
     }
@@ -133,14 +135,14 @@ void KHTMLPartBrowserExtension::editableWidgetBlurred( QWidget * /*widget*/ )
 
     if ( m_connectedToClipboard )
     {
-        disconnect( QApplication::clipboard(), SIGNAL( dataChanged() ),
-                    this, SLOT( updateEditActions() ) );
+        disconnect( QApplication::clipboard(), SIGNAL(dataChanged()),
+                    this, SLOT(updateEditActions()) );
 
         if ( oldWidget )
         {
             if ( oldWidget->inherits( "QLineEdit" ) || oldWidget->inherits( "QTextEdit" ) )
-                disconnect( oldWidget, SIGNAL( selectionChanged() ),
-                            this, SLOT( updateEditActions() ) );
+                disconnect( oldWidget, SIGNAL(selectionChanged()),
+                            this, SLOT(updateEditActions()) );
         }
 
         m_connectedToClipboard = false;
@@ -152,14 +154,14 @@ void KHTMLPartBrowserExtension::setExtensionProxy( KParts::BrowserExtension *pro
 {
     if ( m_extensionProxy )
     {
-        disconnect( m_extensionProxy, SIGNAL( enableAction( const char *, bool ) ),
-                    this, SLOT( extensionProxyActionEnabled( const char *, bool ) ) );
+        disconnect( m_extensionProxy, SIGNAL(enableAction(const char*,bool)),
+                    this, SLOT(extensionProxyActionEnabled(const char*,bool)) );
         if ( m_extensionProxy->inherits( "KHTMLPartBrowserExtension" ) )
         {
-            disconnect( m_extensionProxy, SIGNAL( editableWidgetFocused() ),
-                        this, SLOT( extensionProxyEditableWidgetFocused() ) );
-            disconnect( m_extensionProxy, SIGNAL( editableWidgetBlurred() ),
-                        this, SLOT( extensionProxyEditableWidgetBlurred() ) );
+            disconnect( m_extensionProxy, SIGNAL(editableWidgetFocused()),
+                        this, SLOT(extensionProxyEditableWidgetFocused()) );
+            disconnect( m_extensionProxy, SIGNAL(editableWidgetBlurred()),
+                        this, SLOT(extensionProxyEditableWidgetBlurred()) );
         }
     }
 
@@ -167,14 +169,14 @@ void KHTMLPartBrowserExtension::setExtensionProxy( KParts::BrowserExtension *pro
 
     if ( m_extensionProxy )
     {
-        connect( m_extensionProxy, SIGNAL( enableAction( const char *, bool ) ),
-                 this, SLOT( extensionProxyActionEnabled( const char *, bool ) ) );
+        connect( m_extensionProxy, SIGNAL(enableAction(const char*,bool)),
+                 this, SLOT(extensionProxyActionEnabled(const char*,bool)) );
         if ( m_extensionProxy->inherits( "KHTMLPartBrowserExtension" ) )
         {
-            connect( m_extensionProxy, SIGNAL( editableWidgetFocused() ),
-                     this, SLOT( extensionProxyEditableWidgetFocused() ) );
-            connect( m_extensionProxy, SIGNAL( editableWidgetBlurred() ),
-                     this, SLOT( extensionProxyEditableWidgetBlurred() ) );
+            connect( m_extensionProxy, SIGNAL(editableWidgetFocused()),
+                     this, SLOT(extensionProxyEditableWidgetFocused()) );
+            connect( m_extensionProxy, SIGNAL(editableWidgetBlurred()),
+                     this, SLOT(extensionProxyEditableWidgetBlurred()) );
         }
 
         enableAction( "cut", m_extensionProxy->isActionEnabled( "cut" ) );
@@ -223,7 +225,7 @@ void KHTMLPartBrowserExtension::copy()
         //kDebug(6050) << text;
 
         QClipboard *cb = QApplication::clipboard();
-        disconnect( cb, SIGNAL( selectionChanged() ), m_part, SLOT( slotClearSelection() ) );
+        disconnect( cb, SIGNAL(selectionChanged()), m_part, SLOT(slotClearSelection()) );
 #ifndef QT_NO_MIMECLIPBOARD
   QString htmltext;
   /*
@@ -246,7 +248,7 @@ void KHTMLPartBrowserExtension::copy()
   cb->setText(text);
 #endif
 
-        connect( cb, SIGNAL( selectionChanged() ), m_part, SLOT( slotClearSelection() ) );
+        connect( cb, SIGNAL(selectionChanged()), m_part, SLOT(slotClearSelection()) );
     }
     else
     {
@@ -265,9 +267,8 @@ void KHTMLPartBrowserExtension::searchProvider()
     if (action) {
         KUrl url = action->data().toUrl();
         if (url.host().isEmpty()) {
-            KUriFilterData data;
-            data.setData(action->data().toString());
-            if (KUriFilter::self()->filterUri(data, QStringList() << "kurisearchfilter"))
+            KUriFilterData data(action->data().toString());
+            if (KUriFilter::self()->filterSearchUri(data, KUriFilter::WebShortcutFilter))
                 url = data.uri();
         }
 
@@ -412,7 +413,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
     if (hasSelection) {
         QList<QAction *> editActions;
         QAction* copyAction = d->m_actionCollection->addAction( KStandardAction::Copy, "copy",
-                                                                d->m_khtml->browserExtension(), SLOT( copy() ) );
+                                                                d->m_khtml->browserExtension(), SLOT(copy()) );
 
         copyAction->setText(i18n("&Copy Text"));
         copyAction->setEnabled(d->m_khtml->browserExtension()->isActionEnabled( "copy" ));
@@ -431,7 +432,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
             KAction *action = new KAction(i18n("Open '%1'", selectedTextURL), this);
             d->m_actionCollection->addAction( "openSelection", action );
             action->setIcon( KIcon( "window-new" ) );
-            connect( action, SIGNAL(triggered(bool)), this, SLOT( openSelection() ) );
+            connect( action, SIGNAL(triggered(bool)), this, SLOT(openSelection()) );
             editActions.append(action);
         }
 
@@ -457,7 +458,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
 
             action = new KAction( i18n( "&Copy Link Address" ), this );
             d->m_actionCollection->addAction( "copylinklocation", action );
-            connect( action, SIGNAL(triggered(bool)), this, SLOT( slotCopyLinkLocation() ) );
+            connect( action, SIGNAL(triggered(bool)), this, SLOT(slotCopyLinkLocation()) );
             linkActions.append(action);
         }
         d->actionGroups.insert("linkactions", linkActions);
@@ -476,13 +477,13 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
 
             action = new KAction( i18n( "Open in &This Window" ), this );
             d->m_actionCollection->addAction( "frameintop", action );
-            connect( action, SIGNAL(triggered(bool)), this, SLOT( slotFrameInTop() ) );
+            connect( action, SIGNAL(triggered(bool)), this, SLOT(slotFrameInTop()) );
             menu->addAction(action);
 
             action = new KAction( i18n( "Open in &New Tab" ), this );
             d->m_actionCollection->addAction( "frameintab", action );
             action->setIcon( KIcon( "tab-new" ) );
-            connect( action, SIGNAL(triggered(bool)), this, SLOT( slotFrameInTab() ) );
+            connect( action, SIGNAL(triggered(bool)), this, SLOT(slotFrameInTab()) );
             menu->addAction(action);
 
             action = new KAction(d->m_actionCollection);
@@ -491,28 +492,28 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
 
             action = new KAction( i18n( "Reload Frame" ), this );
             d->m_actionCollection->addAction( "reloadframe", action );
-            connect( action, SIGNAL(triggered(bool)), this, SLOT( slotReloadFrame() ) );
+            connect( action, SIGNAL(triggered(bool)), this, SLOT(slotReloadFrame()) );
             menu->addAction(action);
 
             action = new KAction( i18n( "Print Frame..." ), this );
             d->m_actionCollection->addAction( "printFrame", action );
             action->setIcon( KIcon( "document-print-frame" ) );
-            connect( action, SIGNAL(triggered(bool)), d->m_khtml->browserExtension(), SLOT( print() ) );
+            connect( action, SIGNAL(triggered(bool)), d->m_khtml->browserExtension(), SLOT(print()) );
             menu->addAction(action);
 
             action = new KAction( i18n( "Save &Frame As..." ), this );
             d->m_actionCollection->addAction( "saveFrame", action );
-            connect( action, SIGNAL(triggered(bool)), d->m_khtml, SLOT( slotSaveFrame() ) );
+            connect( action, SIGNAL(triggered(bool)), d->m_khtml, SLOT(slotSaveFrame()) );
             menu->addAction(action);
 
             action = new KAction( i18n( "View Frame Source" ), this );
             d->m_actionCollection->addAction( "viewFrameSource", action );
-            connect( action, SIGNAL(triggered(bool)), d->m_khtml, SLOT( slotViewDocumentSource() ) );
+            connect( action, SIGNAL(triggered(bool)), d->m_khtml, SLOT(slotViewDocumentSource()) );
             menu->addAction(action);
 
             action = new KAction( i18n( "View Frame Information" ), this );
             d->m_actionCollection->addAction( "viewFrameInfo", action );
-            connect( action, SIGNAL(triggered(bool)), d->m_khtml, SLOT( slotViewPageInfo() ) );
+            connect( action, SIGNAL(triggered(bool)), d->m_khtml, SLOT(slotViewPageInfo()) );
 
             action = new KAction(d->m_actionCollection);
             action->setSeparator(true);
@@ -522,7 +523,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
                 if ( khtml->d->m_frame->m_type == khtml::ChildFrame::IFrame ) {
                     action = new KAction( i18n( "Block IFrame..." ), this );
                     d->m_actionCollection->addAction( "blockiframe", action );
-                    connect( action, SIGNAL(triggered(bool)), this, SLOT( slotBlockIFrame() ) );
+                    connect( action, SIGNAL(triggered(bool)), this, SLOT(slotBlockIFrame()) );
                     menu->addAction(action);
                 }
             }
@@ -547,26 +548,26 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
             d->m_imageURL = KUrl( static_cast<DOM::HTMLInputElement>( e ).src().string() );
         KAction *action = new KAction( i18n( "Save Image As..." ), this );
         d->m_actionCollection->addAction( "saveimageas", action );
-        connect( action, SIGNAL(triggered(bool)), this, SLOT( slotSaveImageAs() ) );
+        connect( action, SIGNAL(triggered(bool)), this, SLOT(slotSaveImageAs()) );
         partActions.append(action);
 
         action = new KAction( i18n( "Send Image..." ), this );
         d->m_actionCollection->addAction( "sendimage", action );
-        connect( action, SIGNAL(triggered(bool)), this, SLOT( slotSendImage() ) );
+        connect( action, SIGNAL(triggered(bool)), this, SLOT(slotSendImage()) );
         partActions.append(action);
 
 #ifndef QT_NO_MIMECLIPBOARD
         action = new KAction( i18n( "Copy Image" ), this );
         d->m_actionCollection->addAction( "copyimage", action );
         action->setEnabled(!d->m_pixmap.isNull());
-        connect( action, SIGNAL(triggered(bool)), this, SLOT( slotCopyImage() ) );
+        connect( action, SIGNAL(triggered(bool)), this, SLOT(slotCopyImage()) );
         partActions.append(action);
 #endif
 
         if(d->m_pixmap.isNull()) {    //fallback to image location if still loading the image.  this will always be true if ifdef QT_NO_MIMECLIPBOARD
             action = new KAction( i18n( "Copy Image Location" ), this );
             d->m_actionCollection->addAction( "copyimagelocation", action );
-            connect( action, SIGNAL(triggered(bool)), this, SLOT( slotCopyImageLocation() ) );
+            connect( action, SIGNAL(triggered(bool)), this, SLOT(slotCopyImageLocation()) );
             partActions.append(action);
         }
 
@@ -575,13 +576,13 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
                                    : d->m_suggestedFilename;
         action = new KAction( i18n("View Image (%1)", actionText.replace("&", "&&")), this );
         d->m_actionCollection->addAction( "viewimage", action );
-        connect( action, SIGNAL(triggered(bool)), this, SLOT( slotViewImage() ) );
+        connect( action, SIGNAL(triggered(bool)), this, SLOT(slotViewImage()) );
         partActions.append(action);
 
         if (KHTMLGlobal::defaultHTMLSettings()->isAdFilterEnabled()) {
             action = new KAction( i18n( "Block Image..." ), this );
             d->m_actionCollection->addAction( "blockimage", action );
-            connect( action, SIGNAL(triggered(bool)), this, SLOT( slotBlockImage() ) );
+            connect( action, SIGNAL(triggered(bool)), this, SLOT(slotBlockImage()) );
             partActions.append(action);
 
             if (!d->m_imageURL.host().isEmpty() &&
@@ -589,7 +590,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
             {
                 action = new KAction( i18n( "Block Images From %1" , d->m_imageURL.host()), this );
                 d->m_actionCollection->addAction( "blockhost", action );
-                connect( action, SIGNAL(triggered(bool)), this, SLOT( slotBlockHost() ) );
+                connect( action, SIGNAL(triggered(bool)), this, SLOT(slotBlockHost()) );
                 partActions.append(action);
             }
         }
@@ -628,14 +629,13 @@ void KHTMLPopupGUIClient::addSearchActions(QList<QAction *>& editActions)
     if (selectedText.isEmpty())
         return;
 
-    KUriFilterData data;
-    data.setData(selectedText);
+    KUriFilterData data(selectedText);
     QStringList alternateProviders;
     alternateProviders << "google" << "google_groups" << "google_news" << "webster" << "dmoz" << "wikipedia";
     data.setAlternateSearchProviders(alternateProviders);
     data.setAlternateDefaultSearchProvider("google");
 
-    if (KUriFilter::self()->filterUri(data, QStringList() << "kuriikwsfilter")) {
+    if (KUriFilter::self()->filterSearchUri(data, KUriFilter::NormalTextFilter)) {
         const QString squeezedText = KStringHandler::rsqueeze(selectedText, 21);
         KAction *action = new KAction(i18n("Search for '%1' with %2",
                                            squeezedText, data.searchProvider()), this);
@@ -869,7 +869,8 @@ void KHTMLPopupGUIClient::saveURL( QWidget *parent, const QString &caption,
   int query;
   do {
     query = KMessageBox::Yes;
-    destURL = KFileDialog::getSaveUrl( name, filter, parent, caption );
+    // convert filename to URL using fromPath to avoid trouble with ':' in filenames (#184202)
+    destURL = KFileDialog::getSaveUrl( KUrl::fromPath(name), filter, parent, caption );
       if( destURL.isLocalFile() )
       {
         QFileInfo info( destURL.toLocalFile() );
@@ -991,7 +992,7 @@ bool KHTMLPartBrowserHostExtension::openUrlInFrame(const KUrl &url, const KParts
 KParts::BrowserHostExtension* KHTMLPartBrowserHostExtension::findFrameParent( KParts::ReadOnlyPart
       *callingPart, const QString &frame )
 {
-    KHTMLPart *parentPart = m_part->findFrameParent(callingPart, frame);
+    KHTMLPart *parentPart = m_part->d->findFrameParent(callingPart, frame, 0, true /* navigation*/);
     if (parentPart)
        return parentPart->browserHostExtension();
     return 0;
@@ -1035,7 +1036,7 @@ void KHTMLZoomFactorAction::init(KHTMLPart *part, bool direction)
         addAction( i18n( "%1%" ,  fastZoomSizes[ofs + i] ) );
     }
 
-    connect( selectableActionGroup(), SIGNAL( triggered(QAction*) ), this, SLOT( slotTriggered(QAction*) ) );
+    connect( selectableActionGroup(), SIGNAL(triggered(QAction*)), this, SLOT(slotTriggered(QAction*)) );
 }
 
 KHTMLZoomFactorAction::~KHTMLZoomFactorAction()
@@ -1053,5 +1054,231 @@ void KHTMLZoomFactorAction::slotTriggered(QAction* action)
     setCurrentAction( 0L );
 }
 
-#include "khtml_ext.moc"
+KHTMLTextExtension::KHTMLTextExtension(KHTMLPart* part)
+    : KParts::TextExtension(part)
+{
+    connect(part, SIGNAL(selectionChanged()), this, SIGNAL(selectionChanged()));
+}
 
+KHTMLPart* KHTMLTextExtension::part() const
+{
+    return static_cast<KHTMLPart*>(parent());
+}
+
+bool KHTMLTextExtension::hasSelection() const
+{
+    return part()->hasSelection();
+}
+
+QString KHTMLTextExtension::selectedText(Format format) const
+{
+    switch(format) {
+    case PlainText:
+        return part()->selectedText();
+    case HTML:
+        return part()->selectedTextAsHTML();
+    }
+    return QString();
+}
+
+QString KHTMLTextExtension::completeText(Format format) const
+{
+    switch(format) {
+    case PlainText:
+        return part()->htmlDocument().body().innerText().string();
+    case HTML:
+        return part()->htmlDocument().body().innerHTML().string();
+    }
+    return QString();
+}
+
+////
+
+KHTMLHtmlExtension::KHTMLHtmlExtension(KHTMLPart* part)
+    : KParts::HtmlExtension(part)
+{
+}
+
+KUrl KHTMLHtmlExtension::baseUrl() const
+{
+    return part()->baseURL();
+}
+
+bool KHTMLHtmlExtension::hasSelection() const
+{
+    return part()->hasSelection();
+}
+
+KParts::SelectorInterface::QueryMethods KHTMLHtmlExtension::supportedQueryMethods() const
+{
+    return (KParts::SelectorInterface::SelectedContent | KParts::SelectorInterface::EntireContent);
+}
+
+static KParts::SelectorInterface::Element convertDomElement(const DOM::ElementImpl* domElem)
+{
+    KParts::SelectorInterface::Element elem;
+    elem.setTagName(domElem->tagName().string());
+    const DOM::NamedAttrMapImpl* attrMap = domElem->attributes(true /*readonly*/);
+    if (attrMap) {
+        for (unsigned i = 0; i < attrMap->length(); ++i) {
+            const DOM::AttributeImpl& attr = attrMap->attributeAt(i);
+            elem.setAttribute(attr.localName().string(), attr.value().string());
+            // we could have a setAttributeNS too.
+        }
+    }
+    return elem;
+}
+
+KParts::SelectorInterface::Element KHTMLHtmlExtension::querySelector(const QString& query, KParts::SelectorInterface::QueryMethod method) const
+{
+    KParts::SelectorInterface::Element element;
+
+    // If the specified method is None, return an empty list; similarly
+    // if the document is null, which may be possible in case of an error
+    if (method == KParts::SelectorInterface::None || part()->document().isNull())
+        return element;
+
+    if (!(supportedQueryMethods() & method))
+        return element;
+
+    switch (method) {
+    case KParts::SelectorInterface::EntireContent: {
+        int ec = 0; // exceptions are ignored
+        WTF::RefPtr<DOM::ElementImpl> domElem = part()->document().handle()->querySelector(query, ec);
+        element = convertDomElement(domElem.get());
+        break;
+    }
+    case KParts::SelectorInterface::SelectedContent:
+        if (part()->hasSelection()) {
+            DOM::Element domElem = part()->selection().cloneContents().querySelector(query);
+            element = convertDomElement(static_cast<DOM::ElementImpl*>(domElem.handle()));
+        }
+        break;
+    default:
+        break;
+    }
+
+    return element;
+}
+
+QList<KParts::SelectorInterface::Element> KHTMLHtmlExtension::querySelectorAll(const QString& query, KParts::SelectorInterface::QueryMethod method) const
+{
+    QList<KParts::SelectorInterface::Element> elements;
+
+    // If the specified method is None, return an empty list; similarly
+    // if the document is null, which may be possible in case of an error
+    if (method == KParts::SelectorInterface::None || part()->document().isNull())
+        return elements;
+
+    // If the specified method is not supported, return an empty list...
+    if (!(supportedQueryMethods() & method))
+        return elements;
+
+    switch (method) {
+    case KParts::SelectorInterface::EntireContent: {
+        int ec = 0; // exceptions are ignored
+        WTF::RefPtr<DOM::NodeListImpl> nodes = part()->document().handle()->querySelectorAll(query, ec);    
+        const unsigned long len = nodes->length();
+        elements.reserve(len);
+        for (unsigned long i = 0; i < len; ++i) {
+            DOM::NodeImpl* node = nodes->item(i);
+            if (node->isElementNode()) { // should be always true
+                elements.append(convertDomElement(static_cast<DOM::ElementImpl*>(node)));
+            }
+        }
+        break;
+    }
+    case KParts::SelectorInterface::SelectedContent:
+        if (part()->hasSelection()) {
+            DOM::NodeList nodes = part()->selection().cloneContents().querySelectorAll(query);
+            const unsigned long len = nodes.length();
+            for (unsigned long i = 0; i < len; ++i) {
+                DOM::NodeImpl* node = nodes.item(i).handle();
+                if (node->isElementNode())
+                    elements.append(convertDomElement(static_cast<DOM::ElementImpl*>(node)));
+            }
+        }
+        break;
+    default:
+        break;
+    }
+
+    return elements;
+}
+
+QVariant KHTMLHtmlExtension::htmlSettingsProperty(HtmlSettingsInterface::HtmlSettingsType type) const
+{
+    if (part()) {
+        switch (type) {
+        case KParts::HtmlSettingsInterface::AutoLoadImages:
+            return part()->autoloadImages();
+        case KParts::HtmlSettingsInterface::DnsPrefetchEnabled:
+            return (part()->dnsPrefetch() == KHTMLPart::DNSPrefetchEnabled);
+        case KParts::HtmlSettingsInterface::JavaEnabled:
+            return part()->javaEnabled();
+        case KParts::HtmlSettingsInterface::JavascriptEnabled:
+            return part()->jScriptEnabled();
+        case KParts::HtmlSettingsInterface::MetaRefreshEnabled:
+            return part()->metaRefreshEnabled();
+        case KParts::HtmlSettingsInterface::PluginsEnabled:
+            return part()->pluginsEnabled();
+        default:
+            break;
+        }
+    }
+    return QVariant();
+}
+
+bool KHTMLHtmlExtension::setHtmlSettingsProperty(HtmlSettingsInterface::HtmlSettingsType type, const QVariant& value)
+{
+    KHTMLPart* p = part();
+
+    if (p) {
+        switch (type) {
+        case KParts::HtmlSettingsInterface::AutoLoadImages:
+            p->setAutoloadImages(value.toBool());
+            return true;
+        case KParts::HtmlSettingsInterface::DnsPrefetchEnabled:
+            p->setDNSPrefetch((value.toBool() ? KHTMLPart::DNSPrefetchEnabled : KHTMLPart::DNSPrefetchDisabled));
+            return true;
+        case KParts::HtmlSettingsInterface::JavaEnabled:
+            p->setJavaEnabled(value.toBool());
+            return true;
+        case KParts::HtmlSettingsInterface::JavascriptEnabled:
+            p->setJScriptEnabled(value.toBool());
+            return true;
+        case KParts::HtmlSettingsInterface::MetaRefreshEnabled:
+            p->setMetaRefreshEnabled(value.toBool());
+            return true;
+        case KParts::HtmlSettingsInterface::PluginsEnabled:
+            p->setPluginsEnabled(value.toBool());
+            return true;
+        case KParts::HtmlSettingsInterface::UserDefinedStyleSheetURL: {
+            const KUrl url (value.toUrl());
+            if (url.protocol() == QLatin1String("data")) {
+                const QByteArray data (url.encodedPath());
+                if (!data.isEmpty()) {
+                    const int index = data.indexOf(',');
+                    const QByteArray decodedData ((index > -1 ? QByteArray::fromBase64(data.mid(index)) : QByteArray()));
+                    p->setUserStyleSheet(QString::fromUtf8(decodedData.constData(), decodedData.size()));
+                }
+            } else {
+                p->setUserStyleSheet(url);
+            }
+            return true;
+        }
+        default:
+            break; // Unsupported property...
+        }
+    }
+
+    return false;
+}
+
+
+KHTMLPart* KHTMLHtmlExtension::part() const
+{
+    return static_cast<KHTMLPart*>(parent());
+}
+
+#include "khtml_ext.moc"
