@@ -5,7 +5,7 @@
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
  *           (C) 2004, 2005, 2006, 2007 Apple Inc. All rights reserved.
- *           (C) 2005 Maksim Orlovich (maksim@kde.org)
+ *           (C) 2005, 2009, 2010 Maksim Orlovich (maksim@kde.org)
  *           (C) 2006 Allan Sandfeld Jensen (kde@carewolf.com)
  *
  * This library is free software; you can redistribute it and/or
@@ -27,8 +27,6 @@
 #include "dom_nodeimpl.h"
 
 #include <dom/dom_exception.h>
-#include <misc/htmlattrs.h>
-#include <misc/htmltags.h>
 #include "dom_elementimpl.h"
 #include "dom_textimpl.h"
 #include "dom2_eventsimpl.h"
@@ -36,6 +34,7 @@
 #include "dom_nodelistimpl.h"
 #include "xml/dom_position.h"
 #include "xml/dom_selection.h"
+#include "xml/wa_selectors.h"
 #include "dom_restyler.h"
 #include "html/html_objectimpl.h"
 
@@ -387,60 +386,9 @@ unsigned long NodeImpl::nodeIndex() const
     return count;
 }
 
-void NodeImpl::addEventListener(EventName id, EventListener *listener, const bool useCapture)
+DocumentImpl* NodeImpl::eventTargetDocument()
 {
-    switch (id.id()) {
-	case EventImpl::DOMSUBTREEMODIFIED_EVENT:
-	    document()->addListenerType(DocumentImpl::DOMSUBTREEMODIFIED_LISTENER);
-	    break;
-	case EventImpl::DOMNODEINSERTED_EVENT:
-	    document()->addListenerType(DocumentImpl::DOMNODEINSERTED_LISTENER);
-	    break;
-	case EventImpl::DOMNODEREMOVED_EVENT:
-	    document()->addListenerType(DocumentImpl::DOMNODEREMOVED_LISTENER);
-	    break;
-        case EventImpl::DOMNODEREMOVEDFROMDOCUMENT_EVENT:
-	    document()->addListenerType(DocumentImpl::DOMNODEREMOVEDFROMDOCUMENT_LISTENER);
-	    break;
-        case EventImpl::DOMNODEINSERTEDINTODOCUMENT_EVENT:
-	    document()->addListenerType(DocumentImpl::DOMNODEINSERTEDINTODOCUMENT_LISTENER);
-	    break;
-        case EventImpl::DOMATTRMODIFIED_EVENT:
-	    document()->addListenerType(DocumentImpl::DOMATTRMODIFIED_LISTENER);
-	    break;
-        case EventImpl::DOMCHARACTERDATAMODIFIED_EVENT:
-	    document()->addListenerType(DocumentImpl::DOMCHARACTERDATAMODIFIED_LISTENER);
-	    break;
-	default:
-	    break;
-    }
-
-    m_regdListeners.addEventListener(id, listener, useCapture);
-}
-
-void NodeImpl::removeEventListener(EventName id, EventListener *listener, bool useCapture)
-{
-    m_regdListeners.removeEventListener(id, listener, useCapture);
-}
-
-void NodeImpl::setHTMLEventListener(EventName id, EventListener *listener)
-{
-    m_regdListeners.setHTMLEventListener(id, listener);
-}
-
-void NodeImpl::setHTMLEventListener(unsigned id, EventListener *listener)
-{
-    m_regdListeners.setHTMLEventListener(EventName::fromId(id), listener);
-}
-
-EventListener *NodeImpl::getHTMLEventListener(EventName id)
-{
-    return m_regdListeners.getHTMLEventListener(id);
-}
-
-EventListener *NodeImpl::getHTMLEventListener(unsigned id)
-{
-    return m_regdListeners.getHTMLEventListener(EventName::fromId(id));
+    return document();
 }
 
 void NodeImpl::dispatchEvent(EventImpl *evt, int &exceptioncode, bool tempEvent)
@@ -464,14 +412,34 @@ void NodeImpl::dispatchGenericEvent( EventImpl *evt, int &/*exceptioncode */)
     // ### check that type specified
 
     // work out what nodes to send event to
-    QList<NodeImpl*> nodeChain;
+    QList<EventTargetImpl*> nodeChain;
     NodeImpl *n;
 
-    if (inDocument()) {
+    if (evt->target()->eventTargetType() != DOM_NODE) {
+        // The target is the only thing that goes into the chain.
+        nodeChain.prepend(evt->target());
+        evt->target()->ref();
+
+        // ... except, well, load events lie and say their target is the document,
+        // so we patch that up now (since we want it as Window before we got here
+        if (evt->id() == EventImpl::LOAD_EVENT && evt->target()->eventTargetType() == WINDOW)
+            evt->setTarget(document());
+    } if (inDocument()) {
         for (n = this; n; n = n->parentNode()) {
             n->ref();
             nodeChain.prepend(n);
         }
+
+        // If the event isn't a load event, we propagate it up to window as well.
+        // The exclusion is so that things like image load events don't make it
+        // all the way upto window.onload. Meanwhile, the main load event
+        // is dispatched specially, via dispatchWindowEvent, with the case
+        // above doing the necessary fiddling for it.
+        if (evt->id() != EventImpl::LOAD_EVENT) {
+            EventTargetImpl* t = document()->windowEventTarget();
+            t->ref();
+            nodeChain.prepend(t);
+        }
     } else {
         // if node is not in the document just send event to itself
         ref();
@@ -480,9 +448,9 @@ void NodeImpl::dispatchGenericEvent( EventImpl *evt, int &/*exceptioncode */)
 
     // trigger any capturing event handlers on our way down
     evt->setEventPhase(Event::CAPTURING_PHASE);
-    QListIterator<NodeImpl*> it(nodeChain);
+    QListIterator<EventTargetImpl*> it(nodeChain);
     while (it.hasNext()) {
-        NodeImpl* cur = it.next();
+        EventTargetImpl* cur = it.next();
         if (cur == this || evt->propagationStopped())
             break;
         evt->setCurrentTarget(cur);
@@ -491,8 +459,8 @@ void NodeImpl::dispatchGenericEvent( EventImpl *evt, int &/*exceptioncode */)
 
     // dispatch to the actual target node
     it.toBack();
-    NodeImpl* curn = it.hasPrevious() ? it.previous() : 0;
-    NodeImpl* propagationSentinel = 0;
+    EventTargetImpl* curn = it.hasPrevious() ? it.previous() : 0;
+    EventTargetImpl* propagationSentinel = 0;
     if (curn && !evt->propagationStopped()) {
         evt->setEventPhase(Event::AT_TARGET);
         evt->setCurrentTarget(curn);
@@ -555,19 +523,25 @@ bool NodeImpl::dispatchHTMLEvent(int _id, bool canBubbleArg, bool cancelableArg)
     return ret;
 }
 
+
 void NodeImpl::dispatchWindowEvent(int _id, bool canBubbleArg, bool cancelableArg)
 {
-    int exceptioncode = 0;
     EventImpl* const evt = new EventImpl(static_cast<EventImpl::EventId>(_id),canBubbleArg,cancelableArg);
-    evt->setTarget( 0 );
-    evt->ref();
+    dispatchWindowEvent(evt);
+}
+
+void NodeImpl::dispatchWindowEvent(EventImpl* evt)
+{
     DocumentImpl *doc = document();
     doc->ref();
+    
+    int exceptioncode = 0;
+    
+    evt->setTarget( doc->windowEventTarget() );
+    evt->ref();
     dispatchGenericEvent( evt, exceptioncode );
-    if (!evt->defaultPrevented())
-	doc->defaultEventHandler(evt);
 
-    if (_id == EventImpl::LOAD_EVENT) {
+    if (evt->id() == EventImpl::LOAD_EVENT) {
         // Trigger Load Event on the enclosing frame if there is one
         DOM::HTMLPartContainerElementImpl* elt = doc->ownerElement();
         if (elt)
@@ -697,47 +671,6 @@ bool NodeImpl::dispatchKeyEvent(QKeyEvent *key, bool keypress)
     return r;
 }
 
-void NodeImpl::handleLocalEvents(EventImpl *evt, bool useCapture)
-{
-    if (!m_regdListeners.listeners)
-        return;
-
-    Event ev = evt;
-    // removeEventListener (e.g. called from a JS event listener) might
-    // invalidate the item after the current iterator (which "it" is pointing to).
-    // So we make a copy of the list.
-    QList<RegisteredEventListener> listeners = *m_regdListeners.listeners;
-    QList<RegisteredEventListener>::iterator it;
-    for (it = listeners.begin(); it != listeners.end(); ++it) {
-        //Check whether this got removed...KDE4: use Java-style iterators
-        if (!m_regdListeners.stillContainsListener(*it))
-            continue;
-
-        RegisteredEventListener& current = (*it);
-        if (current.eventName == evt->name() && current.useCapture == useCapture)
-            current.listener->handleEvent(ev);
-
-        // ECMA legacy hack
-        if (current.useCapture == useCapture && evt->id() == EventImpl::CLICK_EVENT) {
-            MouseEventImpl* me = static_cast<MouseEventImpl*>(evt);
-            if (me->button() == 0) {
-                // To find whether to call onclick or ondblclick, we can't
-                // * use me->detail(), it's 2 when clicking twice w/o moving, even very slowly
-                // * use me->qEvent(), it's not available when using initMouseEvent/dispatchEvent
-                // So we currently store a bool in MouseEventImpl. If anyone needs to trigger
-                // dblclicks from the DOM API, we'll need a timer here (well in the doc).
-                if ( ( !me->isDoubleClick() && current.eventName.id() == EventImpl::KHTML_ECMA_CLICK_EVENT) ||
-                  ( me->isDoubleClick() && current.eventName.id() == EventImpl::KHTML_ECMA_DBLCLICK_EVENT) )
-                    current.listener->handleEvent(ev);
-            }
-        }
-    }
-}
-
-void NodeImpl::defaultEventHandler(EventImpl*)
-{
-}
-
 unsigned long NodeImpl::childNodeCount()
 {
     return 0;
@@ -1271,7 +1204,7 @@ bool NodeImpl::hasAttributes() const
 
 bool NodeImpl::isSupported(const DOMString &feature, const DOMString &version)
 {
-    return DOMImplementationImpl::instance()->hasFeature(feature, version);
+    return DOMImplementationImpl::hasFeature(feature, version);
 }
 
 DocumentImpl* NodeImpl::ownerDocument() const
@@ -1357,13 +1290,36 @@ unsigned NodeImpl::compareDocumentPosition(const DOM::NodeImpl* other)
     
     bool thisAncLogical  = thisAnc->parentNode() == 0;
     bool otherAncLogical = otherAnc->parentNode() == 0;
-    kDebug() << thisAncLogical << otherAncLogical;
+    //kDebug() << thisAncLogical << otherAncLogical;
     
     if (thisAncLogical && otherAncLogical) {
         // First, try to order by nodeType.
         if (thisAnc->nodeType() != otherAnc->nodeType())
             return (thisAnc->nodeType() < otherAnc->nodeType()) ? 
                 Node::DOCUMENT_POSITION_FOLLOWING : Node::DOCUMENT_POSITION_PRECEDING;
+
+        // If both are argument nodes, they have to be in the same element,
+        // as otherwise the first difference would be in two different elements
+        // or above, which would not have logical parents unless they were
+        // disconnected, which would have been handled above.
+        // In this case, order them by their position in the
+        // attribute list. This is helpful for XPath.
+
+        if (thisAnc->nodeType() == Node::ATTRIBUTE_NODE) {
+            const AttrImpl* thisAncAttr  = static_cast<const AttrImpl*>(thisAnc);
+            const AttrImpl* otherAncAttr = static_cast<const AttrImpl*>(otherAnc);
+
+            NamedAttrMapImpl* attrs = thisAncAttr->ownerElement()->attributes();
+
+            unsigned l = attrs->length();
+            for (unsigned i = 0; i < l; ++i) {
+                if (attrs->attrAt(i) == thisAncAttr)
+                    return Node::DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC | Node::DOCUMENT_POSITION_FOLLOWING;
+                if (attrs->attrAt(i) == otherAncAttr)
+                    return Node::DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC | Node::DOCUMENT_POSITION_PRECEDING;
+            }
+            assert(false);
+        }
                 
         // If not, another implementation-specific order.
         return Node::DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC |
@@ -1386,6 +1342,26 @@ unsigned NodeImpl::compareDocumentPosition(const DOM::NodeImpl* other)
     return Node::DOCUMENT_POSITION_PRECEDING;
 }
 
+static NodeImpl* rootForSelectorQuery(DOM::NodeImpl* arg)
+{
+    if (arg->nodeType() == Node::DOCUMENT_TYPE_NODE)
+        return static_cast<DOM::DocumentImpl*>(arg)->documentElement();
+    else
+        return arg;
+}
+
+WTF::PassRefPtr<DOM::ElementImpl>  NodeImpl::querySelector(const DOM::DOMString& query, int& ec)
+{
+    return khtml::SelectorQuery::querySelector(rootForSelectorQuery(this),
+                                               query, ec);
+}
+
+WTF::PassRefPtr<DOM::NodeListImpl> NodeImpl::querySelectorAll(const DOM::DOMString& query, int& ec)
+{
+    return khtml::SelectorQuery::querySelectorAll(rootForSelectorQuery(this),
+                                                  query, ec);
+}
+
 void NodeImpl::setDocument(DocumentImpl* doc)
 {
     if (m_document == doc)
@@ -1407,6 +1383,96 @@ void NodeImpl::setDocument(DocumentImpl* doc)
 #endif
 }
 
+DOM::DOMString DOM::NodeImpl::lookupNamespaceURI(const DOM::DOMString& prefix)
+{
+    //for details see http://www.w3.org/TR/DOM-Level-3-Core/namespaces-algorithms.html#lookupNamespaceURIAlgo
+
+    // check if this is one of the hard defined prefixes
+    PrefixName ppn = PrefixName::fromString(prefix);
+    if (ppn.id() == xmlPrefix)
+        return DOM::DOMString(XML_NAMESPACE);
+    if (ppn.id() == xmlnsPrefix)
+        return DOM::DOMString(XMLNS_NAMESPACE);
+    
+    switch( this->nodeType() ) {
+        case Node::ELEMENT_NODE:
+            if( !this->namespaceURI().isNull() && this->prefix() == prefix ) {
+                return this->namespaceURI();
+            }
+            if( this->hasAttributes() ) {
+                ElementImpl* node = static_cast<ElementImpl*>(this);
+
+                NamedAttrMapImpl* attributes = node->attributes( true /*readonly*/);
+                if (ppn.id() != emptyPrefix) {
+                    LocalName pln = LocalName::fromString(prefix);
+                    PrefixName xmlns = PrefixName::fromId(xmlnsNamespace);
+
+                    DOM::DOMString result = attributes->getValue(pln.id(), xmlns);
+                    if( !result.isNull() ) {
+                        return result;
+                    }
+                } else {
+                    DOM::DOMString result = attributes->getValue(ATTR_XMLNS);
+                    if( !result.isEmpty() ) {
+                        return result;
+                    }
+                }
+            }
+            {
+                NodeImpl* ancestor = findNextElementAncestor(this);
+                if( ancestor ) {
+                    return ancestor->lookupNamespaceURI(prefix);
+                }
+            }
+            return DOM::DOMString();
+
+        case Node::DOCUMENT_NODE:
+        {
+            DocumentImpl* node = static_cast<DocumentImpl*>(this);
+            return node->documentElement()->lookupNamespaceURI(prefix);
+        }
+
+        case Node::ATTRIBUTE_NODE:
+        {
+            NodeImpl* ancestor = this->parentNode();
+            if( ancestor ) {
+                return ancestor->lookupNamespaceURI(prefix);
+            }
+            else {
+                return DOM::DOMString();
+            }
+        }
+
+        case Node::ENTITY_NODE: 
+        case Node::NOTATION_NODE: 
+        case Node::DOCUMENT_TYPE_NODE: 
+        case Node::DOCUMENT_FRAGMENT_NODE: 
+           return DOM::DOMString();
+
+        default:
+        {
+            NodeImpl* ancestor = findNextElementAncestor(this);
+            if( ancestor ) {
+                return ancestor->lookupNamespaceURI(prefix);
+            }
+            return DOM::DOMString();
+        }
+    }
+}
+
+DOM::NodeImpl* DOM::NodeImpl::findNextElementAncestor( NodeImpl* node )
+{
+    NodeImpl* iterator = node->parentNode();
+    while( iterator ) {
+        if( iterator->nodeType() == Node::ELEMENT_NODE ) {
+            return iterator;
+        }
+        iterator = iterator->parentNode();
+    }
+    return 0;
+}
+
+
 //-------------------------------------------------------------------------
 
 NodeBaseImpl::~NodeBaseImpl()
@@ -2188,119 +2254,4 @@ void GenericRONamedNodeMapImpl::addNode(NodeImpl *n)
     m_contents->append(n);
 }
 
-// -----------------------------------------------------------------------------
-
-void RegisteredListenerList::addEventListener(EventName id, EventListener *listener, const bool useCapture)
-{
-    if (!listener)
-	return;
-    RegisteredEventListener rl(id,listener,useCapture);
-    if (!listeners)
-        listeners = new QList<RegisteredEventListener>;
-
-    // if this id/listener/useCapture combination is already registered, do nothing.
-    // the DOM2 spec says that "duplicate instances are discarded", and this keeps
-    // the listener order intact.
-    QList<RegisteredEventListener>::iterator it;
-    for (it = listeners->begin(); it != listeners->end(); ++it)
-        if (*it == rl)
-            return;
-
-    listeners->append(rl);
-}
-
-void RegisteredListenerList::removeEventListener(EventName id, EventListener *listener, bool useCapture)
-{
-    if (!listeners) // nothing to remove
-        return;
-
-    RegisteredEventListener rl(id,listener,useCapture);
-
-    QList<RegisteredEventListener>::iterator it;
-    for (it = listeners->begin(); it != listeners->end(); ++it)
-        if (*it == rl) {
-            listeners->erase(it);
-            return;
-        }
-}
-
-bool RegisteredListenerList::isHTMLEventListener(EventListener* listener)
-{
-    return (listener->eventListenerType() == "_khtml_HTMLEventListener");
-}
-
-void RegisteredListenerList::setHTMLEventListener(EventName name, EventListener *listener)
-{
-    if (!listeners)
-        listeners = new QList<RegisteredEventListener>;
-
-    QList<RegisteredEventListener>::iterator it;
-    if (!listener) {
-        for (it = listeners->begin(); it != listeners->end(); ++it) {
-            if ((*it).eventName == name && isHTMLEventListener((*it).listener)) {
-                listeners->erase(it);
-                break;
-            }
-        }
-        return;
-    }
-
-    // if this event already has a registered handler, insert the new one in
-    // place of the old one, to preserve the order.
-    RegisteredEventListener rl(name,listener,false);
-
-    for (int i = 0; i < listeners->size(); ++i) {
-        const RegisteredEventListener& listener = listeners->at(i);
-        if (listener.eventName == name && isHTMLEventListener(listener.listener)) {
-            listeners->replace(i, rl);
-            return;
-        }
-    }
-
-    listeners->append(rl);
-}
-
-EventListener *RegisteredListenerList::getHTMLEventListener(EventName name)
-{
-    if (!listeners)
-        return 0;
-
-    QList<RegisteredEventListener>::iterator it;
-    for (it = listeners->begin(); it != listeners->end(); ++it)
-        if ((*it).eventName == name && isHTMLEventListener((*it).listener)) {
-            return (*it).listener;
-        }
-    return 0;
-}
-
-bool RegisteredListenerList::hasEventListener(EventName name)
-{
-    if (!listeners)
-        return false;
-
-    QList<RegisteredEventListener>::iterator it;
-    for (it = listeners->begin(); it != listeners->end(); ++it)
-        if ((*it).eventName == name)
-            return true;
-
-    return false;
-}
-
-void RegisteredListenerList::clear()
-{
-    delete listeners;
-    listeners = 0;
-}
-
-bool RegisteredListenerList::stillContainsListener(const RegisteredEventListener& listener)
-{
-    if (!listeners)
-        return false;
-    return listeners->contains(listener);
-}
-
-RegisteredListenerList::~RegisteredListenerList() {
-    delete listeners; listeners = 0;
-}
-
 // kate: indent-width 4; replace-tabs on; tab-width 4; space-indent on; 
