@@ -28,8 +28,6 @@
 #include "html/html_tableimpl.h"
 #include "html/html_headimpl.h"
 #include "rendering/render_object.h"
-#include "misc/htmltags.h"
-#include "misc/htmlattrs.h"
 #include "misc/loader.h"
 
 #include "khtmlview.h"
@@ -537,7 +535,7 @@ void XMLTokenizer::finish()
                 stream.readLine();
             line = stream.readLine();
 
-            for (unsigned long colno = 0; colno < m_handler.errorCol-1; colno++)
+            for (long colno = 0; colno < m_handler.errorCol-1; colno++)
                 errorLocPtr += ' ';
             errorLocPtr += '^';
         }
