@@ -42,7 +42,7 @@
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kde_file.h>
-#include <klibloader.h>
+#include <klibrary.h>
 #include <klocale.h>
 #include <kprotocolmanager.h>
 #include <kprotocolinfo.h>
@@ -89,7 +89,7 @@ IdleSlave::gotInput()
    if (mConn.read( &cmd, data) == -1)
    {
       // Communication problem with slave.
-      kError(7016) << "SlavePool: No communication with slave." << endl;
+      //kError(7016) << "SlavePool: No communication with slave." << endl;
       deleteLater();
    }
    else if (cmd == MSG_SLAVE_ACK)
@@ -145,7 +145,7 @@ IdleSlave::reparseConfiguration()
 }
 
 bool
-IdleSlave::match(const QString &protocol, const QString &host, bool needConnected)
+IdleSlave::match(const QString &protocol, const QString &host, bool needConnected) const
 {
    if (mOnHold || protocol != mProtocol) {
       return false;
@@ -157,21 +157,21 @@ IdleSlave::match(const QString &protocol, const QString &host, bool needConnecte
 }
 
 bool
-IdleSlave::onHold(const KUrl &url)
+IdleSlave::onHold(const KUrl &url) const
 {
    if (!mOnHold) return false;
    return (url == mUrl);
 }
 
 int
-IdleSlave::age(time_t now)
+IdleSlave::age(time_t now) const
 {
    return (int) difftime(now, mBirthDate);
 }
 
 static KLauncher* g_klauncher_self;
 
-#ifndef Q_WS_WIN
+#ifndef USE_KPROCESS_FOR_KIOSLAVES
 KLauncher::KLauncher(int _kdeinitSocket)
   : QObject(0),
     kdeinitSocket(_kdeinitSocket)
@@ -206,10 +206,10 @@ KLauncher::KLauncher()
 
    connect(&mTimer, SIGNAL(timeout()), SLOT(idleTimeout()));
 
-#ifndef Q_WS_WIN
+#ifndef USE_KPROCESS_FOR_KIOSLAVES
    kdeinitNotifier = new QSocketNotifier(kdeinitSocket, QSocketNotifier::Read);
-   connect(kdeinitNotifier, SIGNAL( activated( int )),
-           this, SLOT( slotKDEInitData( int )));
+   connect(kdeinitNotifier, SIGNAL(activated(int)),
+           this, SLOT(slotKDEInitData(int)));
    kdeinitNotifier->setEnabled( true );
 #endif
    lastRequest = 0;
@@ -226,7 +226,7 @@ KLauncher::KLauncher()
       mSlaveValgrindSkin = QString::fromLocal8Bit(qgetenv("KDE_SLAVE_VALGRIND_SKIN"));
       qWarning("Klauncher running slaves through valgrind for slaves of protocol '%s'", qPrintable(mSlaveValgrind));
    }
-#ifdef Q_WS_WIN
+#ifdef USE_KPROCESS_FOR_KIOSLAVES
    kDebug(7016) << "LAUNCHER_OK";
 #else
    klauncher_header request_header;
@@ -264,9 +264,7 @@ KLauncher::destruct()
 
 void KLauncher::setLaunchEnv(const QString &name, const QString &value)
 {
-#ifdef Q_WS_WIN
-
-#else
+#ifndef USE_KPROCESS_FOR_KIOSLAVES
    klauncher_header request_header;
    QByteArray requestData;
    requestData.append(name.toLocal8Bit()).append('\0').append(value.toLocal8Bit()).append('\0');
@@ -274,10 +272,13 @@ void KLauncher::setLaunchEnv(const QString &name, const QString &value)
    request_header.arg_length = requestData.size();
    write(kdeinitSocket, &request_header, sizeof(request_header));
    write(kdeinitSocket, requestData.data(), request_header.arg_length);
+#else
+   Q_UNUSED(name);
+   Q_UNUSED(value);
 #endif
 }
 
-#ifndef Q_WS_WIN
+#ifndef USE_KPROCESS_FOR_KIOSLAVES
 /*
  * Read 'len' bytes from 'sock' into buffer.
  * returns -1 on failure, 0 on no data.
@@ -318,11 +319,12 @@ read_socket(int sock, char *buffer, int len)
   }
   return 0;
 }
-
+#endif
 
 void
 KLauncher::slotKDEInitData(int)
 {
+#ifndef USE_KPROCESS_FOR_KIOSLAVES
    klauncher_header request_header;
    QByteArray requestData;
 
@@ -340,9 +342,8 @@ KLauncher::slotKDEInitData(int)
                         request_header.arg_length);
 
    processRequestReturn(request_header.cmd,requestData);
-
-}
 #endif
+}
 
 void KLauncher::processRequestReturn(int status, const QByteArray &requestData)
 {
@@ -560,7 +561,7 @@ KLauncher::requestDone(KLaunchRequest *request)
    {
       requestResult.result = 1;
       requestResult.dbusName.clear();
-      requestResult.error = i18n("KDEInit could not launch '%1'.", request->name);
+      requestResult.error = i18n("KDEInit could not launch '%1'", request->name);
       if (!request->errorMsg.isEmpty())
           requestResult.error += QString::fromLatin1(":\n") + request->errorMsg;
       requestResult.pid = 0;
@@ -621,14 +622,14 @@ static void appendLong(QByteArray &ba, long l)
 void
 KLauncher::requestStart(KLaunchRequest *request)
 {
-#ifdef Q_WS_WIN
+#ifdef USE_KPROCESS_FOR_KIOSLAVES
    requestList.append( request );
    lastRequest = request;
 
    KProcess *process  = new KProcess;
    process->setOutputChannelMode(KProcess::MergedChannels);
    connect(process ,SIGNAL(readyReadStandardOutput()),this, SLOT(slotGotOutput()) );
-   connect(process ,SIGNAL(finished(int, QProcess::ExitStatus)),this, SLOT(slotFinished(int, QProcess::ExitStatus)) );
+   connect(process ,SIGNAL(finished(int,QProcess::ExitStatus)),this, SLOT(slotFinished(int,QProcess::ExitStatus)) );
    request->process = process;
 
 // process.setEnvironment(envlist);
@@ -636,7 +637,13 @@ KLauncher::requestStart(KLaunchRequest *request)
    foreach (const QString &arg, request->arg_list)
       args << arg;
 
-   process->setProgram(request->name,args);
+   QString executable = request->name;
+#ifdef Q_WS_MAC
+   const QString bundlepath = KStandardDirs::findExe(executable);
+   if (!bundlepath.isEmpty())
+      executable = bundlepath;
+#endif
+   process->setProgram(executable,args);
    process->start();
 
    if (!process->waitForStarted())
@@ -729,7 +736,9 @@ KLauncher::start_service_by_name(const QString &serviceName, const QStringList &
 {
    KService::Ptr service;
    // Find service
+#ifndef KDE_NO_DEPRECATED
    service = KService::serviceByName(serviceName);
+#endif
    if (!service)
    {
       requestResult.result = ENOENT;
@@ -746,7 +755,8 @@ KLauncher::start_service_by_desktop_path(const QString &serviceName, const QStri
 {
    KService::Ptr service;
    // Find service
-   if (QFileInfo(serviceName).isAbsolute() && QFile::exists(serviceName))
+   const QFileInfo fi(serviceName);
+   if (fi.isAbsolute() && fi.exists())
    {
       // Full path
       service = new KService(serviceName);
@@ -1012,7 +1022,7 @@ KLauncher::queueRequest(KLaunchRequest *request)
    if (!bProcessingQueue)
    {
       bProcessingQueue = true;
-      QTimer::singleShot(0, this, SLOT( slotDequeue() ));
+      QTimer::singleShot(0, this, SLOT(slotDequeue()));
    }
 }
 
@@ -1074,7 +1084,6 @@ KLauncher::requestHoldSlave(const KUrl &url, const QString &app_socket)
     return 0;
 }
 
-
 pid_t
 KLauncher::requestSlave(const QString &protocol,
                         const QString &host,
@@ -1127,7 +1136,7 @@ KLauncher::requestSlave(const QString &protocol,
     }
 
     QStringList arg_list;
-#ifdef Q_WS_WIN
+#ifdef USE_KPROCESS_FOR_KIOSLAVES
     arg_list << name;
     arg_list << protocol;
     arg_list << mConnectionServer.address();
@@ -1146,18 +1155,25 @@ KLauncher::requestSlave(const QString &protocol,
      << " args=" << arg_list << endl;
 
 #ifdef Q_OS_UNIX
-    if (mSlaveDebug == arg1)
+    if (mSlaveDebug == protocol)
     {
+#ifndef USE_KPROCESS_FOR_KIOSLAVES
        klauncher_header request_header;
        request_header.cmd = LAUNCHER_DEBUG_WAIT;
        request_header.arg_length = 0;
        write(kdeinitSocket, &request_header, sizeof(request_header));
+#else
+      name = QString::fromLatin1("gdb");
+#endif
     }
-    if (mSlaveValgrind == arg1)
-    {
-       arg_list.prepend(KLibLoader::findLibrary(name));
+    if (mSlaveValgrind == protocol) {
+#ifndef USE_KPROCESS_FOR_KIOSLAVES // otherwise we've already done this
+       KLibrary lib(name, KGlobal::mainComponent());
+       arg_list.prepend(lib.fileName());
        arg_list.prepend(KStandardDirs::locate("exe", QString::fromLatin1("kioslave")));
+#endif
        name = QString::fromLatin1("valgrind");
+
        if (!mSlaveValgrindSkin.isEmpty()) {
            arg_list.prepend(QLatin1String("--tool=") + mSlaveValgrindSkin);
        } else
@@ -1188,6 +1204,16 @@ KLauncher::requestSlave(const QString &protocol,
     return pid;
 }
 
+bool KLauncher::checkForHeldSlave(const QString &url)
+{
+    Q_FOREACH (const IdleSlave *p, mSlaveList) {
+       if (p->onHold(url)) {
+          return true;
+       }
+    }
+    return false;
+}
+
 void
 KLauncher::waitForSlave(int pid, const QDBusMessage &msg)
 {
@@ -1210,8 +1236,8 @@ KLauncher::acceptSlave()
     mConnectionServer.setNextPendingConnection(&slave->mConn);
     mSlaveList.append(slave);
     connect(slave, SIGNAL(destroyed()), this, SLOT(slotSlaveGone()));
-    connect(slave, SIGNAL(statusUpdate(IdleSlave *)),
-           this, SLOT(slotSlaveStatus(IdleSlave *)));
+    connect(slave, SIGNAL(statusUpdate(IdleSlave*)),
+           this, SLOT(slotSlaveStatus(IdleSlave*)));
     if (!mTimer.isActive())
     {
        mTimer.start(1000*10);
@@ -1269,18 +1295,21 @@ void KLauncher::reparseConfiguration()
       slave->reparseConfiguration();
 }
 
-#ifdef Q_WS_WIN
+
 void
 KLauncher::slotGotOutput()
 {
+#ifdef USE_KPROCESS_FOR_KIOSLAVES
   KProcess *p = static_cast<KProcess *>(sender());
   QByteArray _stdout = p->readAllStandardOutput();
   kDebug(7016) << _stdout.data();
+#endif
 }
 
 void
 KLauncher::slotFinished(int exitCode, QProcess::ExitStatus exitStatus )
 {
+#ifdef USE_KPROCESS_FOR_KIOSLAVES
     KProcess *p = static_cast<KProcess *>(sender());
     kDebug(7016) << "process finished exitcode=" << exitCode << "exitStatus=" << exitStatus;
 
@@ -1300,13 +1329,17 @@ KLauncher::slotFinished(int exitCode, QProcess::ExitStatus exitStatus )
         }
     }
     delete p;
-}
+#else
+   Q_UNUSED(exitCode);
+   Q_UNUSED(exitStatus);
 #endif
+}
+
 
 void KLauncher::terminate_kdeinit()
 {
     kDebug(7016);
-#ifndef Q_WS_WIN
+#ifndef USE_KPROCESS_FOR_KIOSLAVES
     klauncher_header request_header;
     request_header.cmd = LAUNCHER_TERMINATE_KDEINIT;
     request_header.arg_length = 0;
