@@ -948,12 +948,12 @@ void FunctionBodyNode::compile(CodeType ctype, CompileType compType)
   m_tearOffAtEnd = comp.needsClosures();
 
 #if 0
-  printf("\n\n");
-  printf("\n---------------------------------\n\n");
-  printf("%s", toString().ascii());
-  printf("\n---------------------------------\n\n");
+  fprintf(stderr, "\n\n");
+  fprintf(stderr, "\n---------------------------------\n\n");
+  fprintf(stderr, "%s", toString().ascii());
+  fprintf(stderr, "\n---------------------------------\n\n");
   CodeGen::disassembleBlock(m_compiledCode);
-  printf("\n---------------------------------\n\n");
+  fprintf(stderr, "\n---------------------------------\n\n");
 #endif
 }
 
@@ -998,7 +998,8 @@ FunctionImp* FuncDeclNode::makeFunctionObject(ExecState *exec)
 
   JSObject *proto = exec->lexicalInterpreter()->builtinObject()->construct(exec, List::empty());
   proto->put(exec, exec->propertyNames().constructor, func, DontEnum);
-  func->put(exec, exec->propertyNames().prototype, proto, Internal|DontDelete);
+  // ECMA Edition 5.1r6 - 15.3.5.2 - [[Writable]]: true, [[Enumerable]]: false, [[Configurable]]: false
+  func->put(exec, exec->propertyNames().prototype, proto, Internal|DontDelete|DontEnum);
 
   func->put(exec, exec->propertyNames().length, jsNumber(body->numParams()), ReadOnly|DontDelete|DontEnum);
 
