@@ -35,54 +35,10 @@
 
 #include <typeinfo>
 
-// maximum global call stack size. Protects against accidental or
-// malicious infinite recursions. Define to -1 if you want no limit.
-#if PLATFORM(DARWIN)
-// Given OS X stack sizes we run out of stack at about 350 levels.
-// If we improve our stack usage, we can bump this number.
-#define KJS_MAX_STACK 100
-#else
-#define KJS_MAX_STACK 700 // ### set system specific
-#endif
-
-
 #define JAVASCRIPT_MARK_TRACING 0
 
 namespace KJS {
 
-// ------------------------------ Object ---------------------------------------
-
-JSValue *JSObject::call(ExecState *exec, JSObject *thisObj, const List &args)
-{
-  assert(implementsCall());
-
-#if KJS_MAX_STACK > 0
-  static int depth = 0; // sum of all concurrent interpreters
-
-  if (++depth > KJS_MAX_STACK) {
-    depth -= 11; //Give the debugger some room..
-    JSValue *ret = throwError(exec, RangeError, "Maximum call stack size exceeded.");
-    depth += 10; //Put it back..
-    return ret;
-  }
-#endif
-
-  JSValue *ret = callAsFunction(exec,thisObj,args);
-
-#ifndef NDEBUG
-  if (!ret) {
-    fprintf(stderr, "callAsFunction returned 0 on:%s\n", typeid(*this).name());
-    assert(ret);
-  }
-#endif
-
-#if KJS_MAX_STACK > 0
-  --depth;
-#endif
-
-  return ret;
-}
-
 // ------------------------------ JSObject ------------------------------------
 
 void JSObject::mark()
@@ -129,8 +85,11 @@ JSValue *JSObject::get(ExecState *exec, const Identifier &propertyName) const
 {
   PropertySlot slot;
 
-  if (const_cast<JSObject *>(this)->getPropertySlot(exec, propertyName, slot))
-    return slot.getValue(exec, const_cast<JSObject *>(this), propertyName);
+  if (const_cast<JSObject *>(this)->getPropertySlot(exec, propertyName, slot)) {
+	JSValue* val = slot.getValue(exec, const_cast<JSObject *>(this), propertyName);
+	assert(val);
+	return val;
+  }
 
   return jsUndefined();
 }
@@ -223,12 +182,16 @@ void JSObject::put(ExecState *exec, const Identifier &propertyName, JSValue *val
   if (checkRO) {
     // Check for static properties that are ReadOnly; the property map will check the dynamic properties.
     // We don't have to worry about setters being read-only as they can't be added with such an attribute.
+    // We also need to inherit any attributes we have from the entry
     const HashEntry* entry = findPropertyHashEntry(propertyName);
-    if (entry && entry->attr & ReadOnly) {
+    if (entry) {
+      if (entry->attr & ReadOnly) {
 #ifdef KJS_VERBOSE
-      fprintf( stderr, "WARNING: static property %s is ReadOnly\n", propertyName.ascii() );
+        fprintf( stderr, "WARNING: static property %s is ReadOnly\n", propertyName.ascii() );
 #endif
-      return;
+        return;
+      }
+      attr = entry->attr;
     }
   }
 
@@ -449,6 +412,11 @@ JSObject* JSObject::construct(ExecState* exec, const List& args, const Identifie
   return construct(exec, args);
 }
 
+JSObject* JSObject::valueClone(Interpreter* /*targetCtx*/) const
+{
+    return 0;
+}
+
 bool JSObject::isFunctionType() const
 {
   return implementsCall();
@@ -509,9 +477,9 @@ bool JSObject::getPropertyAttributes(const Identifier& propertyName, unsigned& a
   return false;
 }
 
-void JSObject::getOwnPropertyNames(ExecState* exec, PropertyNameArray& propertyNames)
+void JSObject::getOwnPropertyNames(ExecState* /*exec*/, PropertyNameArray& propertyNames, PropertyMap::PropertyMode mode)
 {
-   _prop.getEnumerablePropertyNames(propertyNames);
+  _prop.getPropertyNames(propertyNames, mode);
 
   // Add properties from the static hashtable of properties
   const ClassInfo *info = classInfo();
@@ -520,7 +488,7 @@ void JSObject::getOwnPropertyNames(ExecState* exec, PropertyNameArray& propertyN
       int size = info->propHashTable->size;
       const HashEntry *e = info->propHashTable->entries;
       for (int i = 0; i < size; ++i, ++e) {
-        if (e->s && !(e->attr & DontEnum))
+        if (e->s && PropertyMap::checkEnumerable(e->attr, mode))
           propertyNames.add(e->s);
       }
     }
