@@ -40,13 +40,58 @@
 
 using namespace KNS3;
 
-void UploadDialog::Private::init()
+bool UploadDialog::Private::init(const QString& configfile)
 {
     QWidget* _mainWidget = new QWidget(q);
     q->setMainWidget(_mainWidget);
     ui.setupUi(_mainWidget);
-
     atticaHelper = new AtticaHelper(q);
+
+    bool success = true;
+    KConfig conf(configfile);
+    if (conf.accessMode() == KConfig::NoAccess) {
+        kError() << "No knsrc file named '" << configfile << "' was found." << endl;
+        success = false;
+    }
+    // FIXME: accessMode() doesn't return NoAccess for non-existing files
+    // - bug in kdecore?
+    // - this needs to be looked at again until KConfig backend changes for KDE 4
+    //   the check below is a workaround
+    if (KStandardDirs::locate("config", configfile).isEmpty()) {
+        kError() << "No knsrc file named '" << configfile << "' was found." << endl;
+        success = false;
+    }
+
+    KConfigGroup group;
+    if (conf.hasGroup("KNewStuff3")) {
+        kDebug() << "Loading KNewStuff3 config: " << configfile;
+        group = conf.group("KNewStuff3");
+    } else {
+        kError() << "A knsrc file was found but it doesn't contain a KNewStuff3 section." << endl;
+        success = false;
+    }
+
+    if ( success ) {
+        const QString providersFileUrl = group.readEntry("ProvidersUrl", QString());
+
+        categoryNames = group.readEntry("UploadCategories", QStringList());
+        // fall back to download categories
+        if (categoryNames.isEmpty()) {
+            categoryNames = group.readEntry("Categories", QStringList());
+        }
+
+        atticaHelper->addProviderFile(QUrl(providersFileUrl));
+    }
+
+    ui.mCategoryCombo->addItems(categoryNames);
+
+    if (categoryNames.size() == 1) {
+        ui.mCategoryLabel->setVisible(false);
+        ui.mCategoryCombo->setVisible(false);
+    }
+
+    kDebug() << "Categories: " << categoryNames;
+
     q->connect(atticaHelper, SIGNAL(providersLoaded(QStringList)), q, SLOT(_k_providersLoaded(QStringList)));
     q->connect(atticaHelper, SIGNAL(loginChecked(bool)), q, SLOT(_k_checkCredentialsFinished(bool)));
     q->connect(atticaHelper, SIGNAL(licensesLoaded(Attica::License::List)), q, SLOT(_k_licensesLoaded(Attica::License::List)));
@@ -73,9 +118,7 @@ void UploadDialog::Private::init()
     ui.busyWidget->layout()->addWidget(busyWidget);
     busyWidget->setVisible(false);
 
-    //ui.previewImage1->showPreview(KUrl("invalid"));
-    //ui.previewImage2->showPreview(KUrl("invalid"));
-    //ui.previewImage3->showPreview(KUrl("invalid"));
+    return success;
 }
 
 void UploadDialog::Private::setBusy(const QString& message)
@@ -358,7 +401,7 @@ UploadDialog::~UploadDialog()
 
 bool UploadDialog::init(const QString &configfile)
 {
-    d->init();
+    bool success = d->init(configfile);
 
     setCaption(i18n("Share Hot New Stuff"));
 
@@ -390,47 +433,11 @@ bool UploadDialog::init(const QString &configfile)
                                  KGlobal::activeComponent().aboutData()->programName()));
     d->ui.mTitleWidget->setPixmap(KIcon(KGlobal::activeComponent().aboutData()->programIconName()));
 
-    KConfig conf(configfile);
-    if (conf.accessMode() == KConfig::NoAccess) {
-        kError() << "No knsrc file named '" << configfile << "' was found." << endl;
-        return false;
-    }
-    // FIXME: accessMode() doesn't return NoAccess for non-existing files
-    // - bug in kdecore?
-    // - this needs to be looked at again until KConfig backend changes for KDE 4
-    // the check below is a workaround
-    if (KStandardDirs::locate("config", configfile).isEmpty()) {
-        kError() << "No knsrc file named '" << configfile << "' was found." << endl;
-        return false;
+    if ( success ) {
+        d->_k_showPage(0);
     }
 
-    KConfigGroup group;
-    if (conf.hasGroup("KNewStuff3")) {
-        kDebug() << "Loading KNewStuff3 config: " << configfile;
-        group = conf.group("KNewStuff3");
-    } else {
-        kError() << "A knsrc file was found but it doesn't contain a KNewStuff3 section." << endl;
-        return false;
-    }
-
-    d->categoryNames = group.readEntry("UploadCategories", QStringList());
-    // fall back to download categories
-    if (d->categoryNames.isEmpty()) {
-        d->categoryNames = group.readEntry("Categories", QStringList());
-    }
-
-    d->ui.mCategoryCombo->addItems(d->categoryNames);
-
-    if (d->categoryNames.size() == 1) {
-        d->ui.mCategoryLabel->setVisible(false);
-        d->ui.mCategoryCombo->setVisible(false);
-    }
-
-    kDebug() << "Categories: " << d->categoryNames;
-
-    d->_k_showPage(0);
-
-    return true;
+    return success;
 }
 
 void UploadDialog::setUploadFile(const KUrl& payloadFile)
