@@ -33,10 +33,6 @@
 #include <plasma/applet.h>
 #include <plasma/animator.h>
 
-namespace KIO
-{
-    class Job;
-}
 
 namespace Plasma
 {
@@ -307,7 +303,7 @@ class PLASMA_EXPORT Containment : public Applet
         /**
          * Sets the open or closed state of the Containment's toolbox
          *
-         * @arg open true to open the ToolBox, false to close it
+         * @param open true to open the ToolBox, false to close it
          */
         void setToolBoxOpen(bool open);
 
@@ -414,7 +410,13 @@ class PLASMA_EXPORT Containment : public Applet
          */
         QString containmentActions(const QString &trigger);
 
-    Q_SIGNALS:
+        /**
+         * @return the config group that containmentactions plugins go in
+         * @since 4.6
+         */
+        KConfigGroup containmentActionsConfig();
+
+Q_SIGNALS:
         /**
          * This signal is emitted when a new applet is created by the containment
          */
@@ -521,7 +523,7 @@ class PLASMA_EXPORT Containment : public Applet
          * it will be removed nicely and deleted.
          * Its configuration will also be deleted.
          *
-         * @arg confirm whether or not confirmation from the user should be requested
+         * @param confirm whether or not confirmation from the user should be requested
          */
         void destroy(bool confirm);
 
@@ -531,7 +533,15 @@ class PLASMA_EXPORT Containment : public Applet
          */
         void showConfigurationInterface();
 
+        /**
+         * Called when applet configuration values have changed.
+         * @reimp
+         * @sa Applet::configChanged()
+         */
+        void configChanged();
+
     protected:
+        //FIXME plasma2: those should be public to allow scripted containments access them
         /**
          * Sets the type of this containment.
          */
@@ -600,7 +610,9 @@ class PLASMA_EXPORT Containment : public Applet
         /**
          * @returns the toolbox associated with this containment, or a null pointer if none
          */
+#ifndef KDE_NO_DEPRECATED
         KDE_DEPRECATED const QGraphicsItem *toolBoxItem() const;
+#endif
 
         /**
          * Sets a custom ToolBox
@@ -632,7 +644,6 @@ class PLASMA_EXPORT Containment : public Applet
         Q_PRIVATE_SLOT(d, void appletDestroyed(Plasma::Applet*))
         Q_PRIVATE_SLOT(d, void appletAppearAnimationComplete())
         Q_PRIVATE_SLOT(d, void triggerShowAddWidgets())
-        Q_PRIVATE_SLOT(d, void handleDisappeared(AppletHandle *handle))
         Q_PRIVATE_SLOT(d, void positionToolBox())
         Q_PRIVATE_SLOT(d, void requestConfiguration())
         Q_PRIVATE_SLOT(d, void updateToolBoxVisibility())
@@ -648,10 +659,12 @@ class PLASMA_EXPORT Containment : public Applet
 
         friend class Applet;
         friend class AppletPrivate;
+        friend class AppletScript;
         friend class CoronaPrivate;
         friend class ContainmentPrivate;
         friend class ContainmentActions;
         friend class PopupApplet;
+        friend class View;
         ContainmentPrivate *const d;
 };
 
